@@ -26,16 +26,17 @@ import six
 import sys
 import traceback
 
-
-from girder.constants import LOG_ROOT, MAX_LOG_SIZE, LOG_BACKUP_COUNT, \
-    TerminalColor
+from girder.constants import LOG_ROOT, MAX_LOG_SIZE, LOG_BACKUP_COUNT, TerminalColor, VERSION
 from girder.utility import config, mkdir
 
-__version__ = '2.2.0'
+__version__ = '2.4.0'
 __license__ = 'Apache 2.0'
 
 
+VERSION['apiVersion'] = __version__
 _quiet = False
+_originalStdOut = sys.stdout
+_originalStdErr = sys.stderr
 
 
 class LogLevelFilter(object):
@@ -70,11 +71,50 @@ class LogFormatter(logging.Formatter):
 
     def format(self, record, *args, **kwargs):
         if hasattr(record, 'name') and hasattr(record, 'message'):
-            if record.name.startswith('cherrypy.access'):
+            if (record.name.startswith('cherrypy.access') or
+                    record.name.startswith('cherrypy.error')):
                 return record.message
         return super(LogFormatter, self).format(record, *args, **kwargs)
 
 
+class StreamToLogger:
+    """
+    Redirect a file-like stream to a logger.
+    """
+    def __init__(self, stream, logger, level):
+        self.stream = stream
+        self.logger = logger
+        self.level = level
+        self.logger._girderLogHandlerOutput = False
+        # This class is intended to override a default stream like sys.stdout
+        # and sys.stderr and send that information to both the original stream
+        # and the logger method.  However, we want to preserve as much
+        # functionality for stdout and stderr as possible, so that other
+        # modules that send data to them can do so without a problem.  The only
+        # method we really need to override is write, but we cannot mutate the
+        # write method on the stream itself, so we replace the stream with this
+        # custom class.  To preserve the stream methods, all of them get added
+        # to our class instance except private and built-in methods, which, in
+        # python, begin with _.
+        #     Fundamentally, this lets our stream replacement handle functions
+        # flush, writeline, and others without having to enumerate them
+        # individually.
+        for key in dir(stream):
+            # It's possible for a file-like object to have name appear in dir(stream) but not
+            # actually be an attribute, thus using a default with getattr is required.
+            # See https://github.com/GrahamDumpleton/mod_wsgi/issues/184 for more.
+            if key != 'write' and not key.startswith('_') and callable(getattr(stream, key, None)):
+                setattr(self, key, getattr(stream, key))
+
+    def write(self, buf):
+        if not self.logger._girderLogHandlerOutput:
+            self.logger._girderLogHandlerOutput = True
+            self.stream.write(buf)
+            for line in buf.rstrip().splitlines():
+                self.logger.log(self.level, line.rstrip())
+            self.logger._girderLogHandlerOutput = False
+
+
 def getLogPaths():
     """
     Return the paths to the error and info log files. These are returned as
@@ -141,6 +181,7 @@ def _setupLogger():
             cherrypy.log.access_log.removeHandler(handler)
 
     fmt = LogFormatter('[%(asctime)s] %(levelname)s: %(message)s')
+    infoMaxLevel = logging.INFO
     # Create log handlers
     if logPaths['error'] != logPaths['info']:
         eh = logging.handlers.RotatingFileHandler(
@@ -150,14 +191,22 @@ def _setupLogger():
         eh._girderLogHandler = 'error'
         eh.setFormatter(fmt)
         logger.addHandler(eh)
+        # Record cherrypy errors in our logs, too
+        cherrypy.log.error_log.addHandler(eh)
+    else:
+        infoMaxLevel = logging.CRITICAL
 
+    if isinstance(getattr(logging, logCfg.get('log_max_info_level', ''), None), int):
+        infoMaxLevel = getattr(logging, logCfg['log_max_info_level'])
     ih = logging.handlers.RotatingFileHandler(
         logPaths['info'], maxBytes=logSize, backupCount=backupCount)
     ih.setLevel(level)
-    ih.addFilter(LogLevelFilter(min=logging.DEBUG, max=logging.INFO))
+    ih.addFilter(LogLevelFilter(min=logging.DEBUG, max=infoMaxLevel))
     ih._girderLogHandler = 'info'
     ih.setFormatter(fmt)
     logger.addHandler(ih)
+    # Record cherrypy errors in our logs, too
+    cherrypy.log.error_log.addHandler(ih)
 
     # Log http accesses to the screen and/or the info log.
     accessLog = logCfg.get('log_access', 'screen')
@@ -174,6 +223,12 @@ def _setupLogger():
 logger = _setupLogger()
 
 
+def logStdoutStderr():
+    if _originalStdOut == sys.stdout:
+        sys.stdout = StreamToLogger(_originalStdOut, logger, logging.INFO)
+        sys.stderr = StreamToLogger(_originalStdErr, logger, logging.ERROR)
+
+
 def logprint(*args, **kwargs):
     """
     Send a message to both stdout and the appropriate logs.  This behaves like
@@ -200,7 +255,8 @@ def logprint(*args, **kwargs):
     if not _quiet:
         if color:
             data = getattr(TerminalColor, color)(data)
-        six.print_(data, flush=True)
+        _originalStdOut.write('%s\n' % data)
+        _originalStdOut.flush()
 
 
 # Expose common logging levels and colors as methods of logprint.
