@@ -24,7 +24,7 @@ from .v1 import api_key, assetstore, file, collection, folder, group, item,\
     resource, system, token, user, notification
 
 
-class ApiDocs():
+class ApiDocs(object):
     exposed = True
 
     def GET(self):
