@@ -19,17 +19,24 @@
 
 import bson.json_util
 import dateutil.parser
+try:
+    from inspect import signature, Parameter
+except ImportError:
+    from funcsigs import signature, Parameter
 import inspect
 import jsonschema
 import os
 import six
 import cherrypy
 
-from girder import constants, logprint
+from girder import constants, events, logprint
 from girder.api.rest import getCurrentUser, RestException, getBodyJson
+from girder.constants import CoreEventHandler, SettingKey
+from girder.models.setting import Setting
 from girder.utility import config, toBool
 from girder.utility.model_importer import ModelImporter
 from girder.utility.webroot import WebrootBase
+from girder.utility.resource import _apiRouteMap
 from . import docs, access
 from .rest import Resource, getApiUrl, getUrlParts
 
@@ -80,9 +87,11 @@ class Description(object):
         self._params = []
         self._responses = {}
         self._consumes = []
+        self._produces = []
         self._responseClass = None
         self._responseClassArray = False
         self._notes = None
+        self._deprecated = False
         self.hasPagingParams = False
         self.modelParams = {}
         self.jsonParams = {}
@@ -124,6 +133,12 @@ class Description(object):
         if self._consumes:
             resp['consumes'] = self._consumes
 
+        if self._produces:
+            resp['produces'] = self._produces
+
+        if self._deprecated:
+            resp['deprecated'] = True
+
         return resp
 
     def responseClass(self, obj, array=False):
@@ -247,7 +262,8 @@ class Description(object):
         and the model should be loaded and passed into the route handler. For example,
         if you have a route like ``GET /item/:id``, you could do:
 
-        >>> modelParam('id', model='item', level=AccessType.READ)
+        >>> from girder.models.item import Item
+        >>> modelParam('id', model=Item, level=AccessType.READ)
 
         Which would cause the ``id`` parameter in the path to be mapped to an
         item model parameter named ``item``, and ensure that the calling user
@@ -255,7 +271,7 @@ class Description(object):
         the query string or form data, for example a request like
         ``POST /item?folderId=...``, you must specify the ``paramType``.
 
-        >>> modelParam('folderId', 'The ID of the parent folder.',
+        >>> modelParam('folderId', 'The ID of the parent folder.', model=Folder,
         ...            level=AccessType.WRITE, paramType='query')
 
         Note that in the above example, ``model`` is omitted; in this case, the
@@ -271,11 +287,12 @@ class Description(object):
         :type destName: str
         :param paramType: how is the parameter sent.  One of 'query', 'path',
             'body', 'header', or 'formData'.
-        :param model: The model name, e.g. 'folder'. If not passed, defaults to stripping
-            the last two characters from the name, such that e.g. 'folderId' would make
-            the model become 'folder'.
-        :type model: str
-        :param plugin: Plugin name, if loading a plugin model.
+        :param model: The model class to use for loading, or a name, e.g. 'folder'. If not passed,
+            defaults to stripping the last two characters from the name, such that e.g. 'folderId'
+            would make the model become 'folder'.
+        :type model: class or str
+        :param plugin: Plugin name, if loading a plugin model. Only used when the ``model``
+            param is a string rather than a class.
         :type plugin: str
         :param level: Access level, if this is an access controlled model.
         :type level: AccessType
@@ -291,17 +308,20 @@ class Description(object):
         if model is None:
             model = name[:-2]  # strip off "Id"
 
+        isModelClass = inspect.isclass(model)
+
         if description is None:
-            description = 'The ID of the %s.' % model
+            description = 'The ID of the document.'
 
         self.param(name=name, description=description, paramType=paramType, required=required)
 
         self.modelParams[name] = {
-            'destName': destName or model,
+            'destName': destName,
             'level': level,
             'force': force,
             'model': model,
             'plugin': plugin,
+            'isModelClass': isModelClass,
             'exc': exc,
             'required': required,
             'requiredFlags': requiredFlags,
@@ -370,6 +390,13 @@ class Description(object):
         self._consumes.append(value)
         return self
 
+    def produces(self, value):
+        if isinstance(value, (list, tuple)):
+            self._produces.extend(value)
+        else:
+            self._produces.append(value)
+        return self
+
     def notes(self, notes):
         self._notes = notes
         return self
@@ -401,6 +428,13 @@ class Description(object):
 
         return self
 
+    def deprecated(self):
+        """
+        Mark the route as deprecated.
+        """
+        self._deprecated = True
+        return self
+
     @property
     def params(self):
         return self._params
@@ -422,10 +456,25 @@ class ApiDocs(WebrootBase):
         self.vars = {
             'apiRoot': '',
             'staticRoot': '',
-            'title': 'Girder - REST API Documentation',
+            'brandName': Setting().get(SettingKey.BRAND_NAME),
             'mode': mode
         }
 
+        events.bind('model.setting.save.after', CoreEventHandler.WEBROOT_SETTING_CHANGE,
+                    self._onSettingSave)
+        events.bind('model.setting.remove', CoreEventHandler.WEBROOT_SETTING_CHANGE,
+                    self._onSettingRemove)
+
+    def _onSettingSave(self, event):
+        settingDoc = event.info
+        if settingDoc['key'] == SettingKey.BRAND_NAME:
+            self.updateHtmlVars({'brandName': settingDoc['value']})
+
+    def _onSettingRemove(self, event):
+        settingDoc = event.info
+        if settingDoc['key'] == SettingKey.BRAND_NAME:
+            self.updateHtmlVars({'brandName': Setting().getDefault(SettingKey.BRAND_NAME)})
+
 
 class Describe(Resource):
     def __init__(self):
@@ -443,15 +492,25 @@ class Describe(Resource):
         # List of Tag Objects
         tags = []
 
-        for resource in sorted(six.viewkeys(docs.routes)):
+        routeMap = _apiRouteMap()
+
+        for resource in sorted(six.viewkeys(docs.routes), key=str):
             # Update Definitions Object
             if resource in docs.models:
                 for name, model in six.viewitems(docs.models[resource]):
                     definitions[name] = model
 
+            prefixPath = None
+            tag = resource
+            if isinstance(resource, Resource):
+                if resource not in routeMap:
+                    raise RestException('Resource not mounted: %s' % resource)
+                prefixPath = routeMap[resource]
+                tag = prefixPath[0]
+
             # Tag Object
             tags.append({
-                'name': resource
+                'name': tag
             })
 
             for route, methods in six.viewitems(docs.routes[resource]):
@@ -460,6 +519,11 @@ class Describe(Resource):
                 for method, operation in six.viewitems(methods):
                     # Operation Object
                     pathItem[method.lower()] = operation
+                    if prefixPath:
+                        operation['tags'] = prefixPath[:1]
+
+                if prefixPath:
+                    route = '/'.join([''] + prefixPath + [route[1:]])
 
                 paths[route] = pathItem
 
@@ -534,16 +598,40 @@ class autoDescribeRoute(describeRoute):  # noqa: class name
         :type kwargs: dict
         :param val: The value of the argument to set
         """
-        if name in fun._fnArgs or fun._fnKeywds is not None:
+        if name in self._funNamedArgs or self._funHasKwargs:
             kwargs[name] = val
             kwargs['params'].pop(name, None)
         else:
             kwargs['params'][name] = val
 
+    def _mungeKwargs(self, kwargs, fun):
+        """
+        Performs final modifications to the kwargs passed into the wrapped function.
+        Combines the sort/sortdir params appropriately for consumption by the model
+        layer, and only passes the "params" catch-all dict if there is a corresponding
+        kwarg for it in the wrapped function.
+        """
+        if self.description.hasPagingParams and 'sort' in kwargs:
+            sortdir = kwargs.pop('sortdir', None) or kwargs['params'].pop('sortdir', None)
+            kwargs['sort'] = [(kwargs['sort'], sortdir)]
+
+        if 'params' not in self._funNamedArgs and not self._funHasKwargs:
+            kwargs.pop('params', None)
+
+    def _inspectFunSignature(self, fun):
+        self._funNamedArgs = set()
+        self._funHasKwargs = False
+        for funParam in six.viewvalues(signature(fun).parameters):
+            if funParam.kind in {Parameter.POSITIONAL_OR_KEYWORD, Parameter.KEYWORD_ONLY}:
+                # POSITIONAL_OR_KEYWORD are basic positional parameters
+                # KEYWORD_ONLY are named parameters that appear after a * in Python 3
+                self._funNamedArgs.add(funParam.name)
+            elif funParam.kind == Parameter.VAR_KEYWORD:
+                # VAR_KEYWORD is the **kwargs parameter
+                self._funHasKwargs = True
+
     def __call__(self, fun):
-        fnInfo = inspect.getargspec(fun)
-        fun._fnArgs = set(fnInfo.args)
-        fun._fnKeywds = fnInfo.keywords
+        self._inspectFunSignature(fun)
 
         @six.wraps(fun)
         def wrapped(*args, **kwargs):
@@ -559,7 +647,9 @@ class autoDescribeRoute(describeRoute):  # noqa: class name
                 # We need either a type or a schema ( for message body )
                 if 'type' not in descParam and 'schema' not in descParam:
                     continue
+
                 name = descParam['name']
+                model = self._getModel(name, self.description.modelParams)
                 if name in params:
                     if name in self.description.jsonParams:
                         info = self.description.jsonParams[name]
@@ -568,8 +658,8 @@ class autoDescribeRoute(describeRoute):  # noqa: class name
                     elif name in self.description.modelParams:
                         info = self.description.modelParams[name]
                         kwargs.pop(name, None)  # Remove from path params
-                        val = self._loadModel(name, info, params[name])
-                        self._passArg(fun, kwargs, info['destName'], val)
+                        val = self._loadModel(name, info, params[name], model)
+                        self._passArg(fun, kwargs, info['destName'] or model.name, val)
                     else:
                         val = self._validateParam(name, descParam, params[name])
                         self._passArg(fun, kwargs, name, val)
@@ -590,13 +680,11 @@ class autoDescribeRoute(describeRoute):  # noqa: class name
                     if name in self.description.modelParams:
                         info = self.description.modelParams[name]
                         kwargs.pop(name, None)  # Remove from path params
-                        self._passArg(fun, kwargs, info['destName'], None)
+                        self._passArg(fun, kwargs, info['destName'] or model.name, None)
                     else:
                         self._passArg(fun, kwargs, name, None)
 
-            if self.description.hasPagingParams and 'sort' in kwargs:
-                sortdir = kwargs.pop('sortdir', None) or kwargs['params'].pop('sortdir', None)
-                kwargs['sort'] = [(kwargs['sort'], sortdir)]
+            self._mungeKwargs(kwargs, fun)
 
             return fun(*args, **kwargs)
 
@@ -638,8 +726,17 @@ class autoDescribeRoute(describeRoute):  # noqa: class name
 
         return val
 
-    def _loadModel(self, name, info, id):
-        model = ModelImporter.model(info['model'], info['plugin'])
+    def _getModel(self, name, modelParams):
+        if name not in self.description.modelParams:
+            return
+        info = self.description.modelParams[name]
+
+        if info['isModelClass']:
+            return info['model']()
+        else:
+            return ModelImporter.model(info['model'], info['plugin'])
+
+    def _loadModel(self, name, info, id, model):
         if info['force']:
             doc = model.load(id, force=True, **info['kwargs'])
         elif info['level'] is not None:
@@ -714,6 +811,6 @@ class autoDescribeRoute(describeRoute):  # noqa: class name
         # Enum validation (should be afer type coercion)
         if 'enum' in descParam and value not in descParam['enum']:
             raise RestException('Invalid value for %s: "%s". Allowed values: %s.' % (
-                name, value, ', '.join(descParam['enum'])))
+                name, value, ', '.join(str(v) for v in descParam['enum'])))
 
         return value
