@@ -22,7 +22,10 @@ from ..rest import Resource, RestException
 from girder import events
 from girder.constants import AccessType, AssetstoreType, TokenScope
 from girder.api import access
+from girder.models.assetstore import Assetstore as AssetstoreModel
+from girder.models.file import File
 from girder.utility.progress import ProgressContext
+from girder.utility.s3_assetstore_adapter import DEFAULT_REGION
 
 
 class Assetstore(Resource):
@@ -32,6 +35,8 @@ class Assetstore(Resource):
     def __init__(self):
         super(Assetstore, self).__init__()
         self.resourceName = 'assetstore'
+        self._model = AssetstoreModel()
+
         self.route('GET', (), self.find)
         self.route('GET', (':id',), self.getAssetstore)
         self.route('POST', (), self.createAssetstore)
@@ -43,12 +48,12 @@ class Assetstore(Resource):
     @access.admin
     @autoDescribeRoute(
         Description('Get information about an assetstore.')
-        .modelParam('id', model='assetstore')
+        .modelParam('id', model=AssetstoreModel)
         .errorResponse()
         .errorResponse('You are not an administrator.', 403)
     )
-    def getAssetstore(self, assetstore, params):
-        self.model('assetstore').addComputedInfo(assetstore)
+    def getAssetstore(self, assetstore):
+        self._model.addComputedInfo(assetstore)
         return assetstore
 
     @access.admin
@@ -58,8 +63,8 @@ class Assetstore(Resource):
         .errorResponse()
         .errorResponse('You are not an administrator.', 403)
     )
-    def find(self, limit, offset, sort, params):
-        return list(self.model('assetstore').list(offset=offset, limit=limit, sort=sort))
+    def find(self, limit, offset, sort):
+        return list(self._model.list(offset=offset, limit=limit, sort=sort))
 
     @access.admin
     @autoDescribeRoute(
@@ -68,16 +73,14 @@ class Assetstore(Resource):
         .notes('You must be an administrator to call this.')
         .param('name', 'Unique name for the assetstore.')
         .param('type', 'Type of the assetstore.', dataType='integer')
-        .param('root', 'Root path on disk (for filesystem type).',
-               required=False)
-        .param('perms', 'File creation permissions (for filesystem type).',
-               required=False)
+        .param('root', 'Root path on disk (for filesystem type).', required=False)
+        .param('perms', 'File creation permissions (for filesystem type).', required=False)
         .param('db', 'Database name (for GridFS type)', required=False)
         .param('mongohost', 'Mongo host URI (for GridFS type)', required=False)
-        .param('replicaset', 'Replica set name (for GridFS type)',
-               required=False)
-        .param('bucket', 'The S3 bucket to store data in (for S3 type).',
-               required=False)
+        .param('replicaset', 'Replica set name (for GridFS type)', required=False)
+        .param('shard', 'Shard the collection (for GridFS type).  Set to '
+               '"auto" to set up sharding.', required=False)
+        .param('bucket', 'The S3 bucket to store data in (for S3 type).', required=False)
         .param('prefix', 'Optional path prefix within the bucket under which '
                'files will be stored (for S3 type).', required=False, default='')
         .param('accessKeyId', 'The AWS access key ID to use for authentication '
@@ -90,24 +93,30 @@ class Assetstore(Resource):
                'Do not include the bucket name here.', required=False, default='')
         .param('readOnly', 'If this assetstore is read-only, set this to true.',
                required=False, dataType='boolean', default=False)
+        .param('region', 'The AWS region to which the S3 bucket belongs.', required=False,
+               default=DEFAULT_REGION)
+        .param('inferCredentials', 'The credentials for connecting to S3 will be inferred '
+               'by Boto rather than explicitly passed. Inferring credentials will '
+               'ignore accessKeyId and secret.', dataType='boolean', required=False)
         .errorResponse()
         .errorResponse('You are not an administrator.', 403)
     )
-    def createAssetstore(self, name, type, root, perms, db, mongohost, replicaset, bucket,
-                         prefix, accessKeyId, secret, service, readOnly, params):
+    def createAssetstore(self, name, type, root, perms, db, mongohost, replicaset, shard, bucket,
+                         prefix, accessKeyId, secret, service, readOnly, region, inferCredentials):
         if type == AssetstoreType.FILESYSTEM:
             self.requireParams({'root': root})
-            return self.model('assetstore').createFilesystemAssetstore(
+            return self._model.createFilesystemAssetstore(
                 name=name, root=root, perms=perms)
         elif type == AssetstoreType.GRIDFS:
             self.requireParams({'db': db})
-            return self.model('assetstore').createGridFsAssetstore(
-                name=name, db=db, mongohost=mongohost, replicaset=replicaset)
+            return self._model.createGridFsAssetstore(
+                name=name, db=db, mongohost=mongohost, replicaset=replicaset, shard=shard)
         elif type == AssetstoreType.S3:
             self.requireParams({'bucket': bucket})
-            return self.model('assetstore').createS3Assetstore(
+            return self._model.createS3Assetstore(
                 name=name, bucket=bucket, prefix=prefix, secret=secret,
-                accessKeyId=accessKeyId, service=service, readOnly=readOnly)
+                accessKeyId=accessKeyId, service=service, readOnly=readOnly, region=region,
+                inferCredentials=inferCredentials)
         else:
             raise RestException('Invalid type parameter')
 
@@ -118,7 +127,7 @@ class Assetstore(Resource):
                'references to it in the Girder data hierarchy. Deleting '
                'those references will not delete the underlying data. This '
                'operation is currently only supported for S3 assetstores.')
-        .modelParam('id', model='assetstore')
+        .modelParam('id', model=AssetstoreModel)
         .param('importPath', 'Root path within the underlying storage system '
                'to import.', required=False)
         .param('destinationId', 'ID of a folder, collection, or user in Girder '
@@ -138,13 +147,13 @@ class Assetstore(Resource):
         .errorResponse('You are not an administrator.', 403)
     )
     def importData(self, assetstore, importPath, destinationId, destinationType, progress,
-                   leafFoldersAsItems, fileIncludeRegex, fileExcludeRegex, params):
+                   leafFoldersAsItems, fileIncludeRegex, fileExcludeRegex):
         user = self.getCurrentUser()
         parent = self.model(destinationType).load(
             destinationId, user=user, level=AccessType.ADMIN, exc=True)
 
         with ProgressContext(progress, user=user, title='Importing data') as ctx:
-            return self.model('assetstore').importData(
+            return self._model.importData(
                 assetstore, parent=parent, parentType=destinationType, params={
                     'fileIncludeRegex': fileIncludeRegex,
                     'fileExcludeRegex': fileExcludeRegex,
@@ -155,13 +164,15 @@ class Assetstore(Resource):
     @autoDescribeRoute(
         Description('Update an existing assetstore.')
         .responseClass('Assetstore')
-        .modelParam('id', model='assetstore')
+        .modelParam('id', model=AssetstoreModel)
         .param('name', 'Unique name for the assetstore.', strip=True)
         .param('root', 'Root path on disk (for Filesystem type)', required=False)
         .param('perms', 'File creation permissions (for Filesystem type)', required=False)
         .param('db', 'Database name (for GridFS type)', required=False)
         .param('mongohost', 'Mongo host URI (for GridFS type)', required=False)
         .param('replicaset', 'Replica set name (for GridFS type)', required=False)
+        .param('shard', 'Shard the collection (for GridFS type).  Set to '
+               '"auto" to set up sharding.', required=False)
         .param('bucket', 'The S3 bucket to store data in (for S3 type).', required=False)
         .param('prefix', 'Optional path prefix within the bucket under which '
                'files will be stored (for S3 type).', required=False, default='')
@@ -175,12 +186,18 @@ class Assetstore(Resource):
                'Do not include the bucket name here.', required=False, default='')
         .param('readOnly', 'If this assetstore is read-only, set this to true.',
                required=False, dataType='boolean')
+        .param('region', 'The AWS region to which the S3 bucket belongs.', required=False,
+               default=DEFAULT_REGION)
         .param('current', 'Whether this is the current assetstore', dataType='boolean')
+        .param('inferCredentials', 'The credentials for connecting to S3 will be inferred '
+               'by Boto rather than explicitly passed. Inferring credentials will '
+               'ignore accessKeyId and secret.', dataType='boolean', required=False)
         .errorResponse()
         .errorResponse('You are not an administrator.', 403)
     )
-    def updateAssetstore(self, assetstore, name, root, perms, db, mongohost, replicaset, bucket,
-                         prefix, accessKeyId, secret, service, readOnly, current, params):
+    def updateAssetstore(self, assetstore, name, root, perms, db, mongohost, replicaset, shard,
+                         bucket, prefix, accessKeyId, secret, service, readOnly, region, current,
+                         inferCredentials, params):
         assetstore['name'] = name
         assetstore['current'] = current
 
@@ -196,17 +213,19 @@ class Assetstore(Resource):
                 assetstore['mongohost'] = mongohost
             if replicaset is not None:
                 assetstore['replicaset'] = replicaset
+            if shard is not None:
+                assetstore['shard'] = shard
         elif assetstore['type'] == AssetstoreType.S3:
             self.requireParams({
-                'bucket': bucket,
-                'accessKeyId': accessKeyId,
-                'secret': secret
+                'bucket': bucket
             })
             assetstore['bucket'] = bucket
             assetstore['prefix'] = prefix
             assetstore['accessKeyId'] = accessKeyId
             assetstore['secret'] = secret
             assetstore['service'] = service
+            assetstore['region'] = region
+            assetstore['inferCredentials'] = inferCredentials
             if readOnly is not None:
                 assetstore['readOnly'] = readOnly
         else:
@@ -214,37 +233,36 @@ class Assetstore(Resource):
                 'assetstore': assetstore,
                 'params': dict(
                     name=name, current=current, readOnly=readOnly, root=root, perms=perms,
-                    db=db, mongohost=mongohost, replicaset=replicaset, bucket=bucket,
+                    db=db, mongohost=mongohost, replicaset=replicaset, shard=shard, bucket=bucket,
                     prefix=prefix, accessKeyId=accessKeyId, secret=secret, service=service,
-                    **params
+                    region=region, **params
                 )
             })
             if event.defaultPrevented:
                 return
-        return self.model('assetstore').save(assetstore)
+        return self._model.save(assetstore)
 
     @access.admin
     @autoDescribeRoute(
         Description('Delete an assetstore.')
         .notes('This will fail if there are any files in the assetstore.')
-        .modelParam('id', model='assetstore')
+        .modelParam('id', model=AssetstoreModel)
         .errorResponse(('A parameter was invalid.',
                         'The assetstore is not empty.'))
         .errorResponse('You are not an administrator.', 403)
     )
-    def deleteAssetstore(self, assetstore, params):
-        self.model('assetstore').remove(assetstore)
+    def deleteAssetstore(self, assetstore):
+        self._model.remove(assetstore)
         return {'message': 'Deleted assetstore %s.' % assetstore['name']}
 
     @access.admin
     @autoDescribeRoute(
         Description('Get a list of files controlled by an assetstore.')
-        .modelParam('id', model='assetstore')
+        .modelParam('id', model=AssetstoreModel)
         .pagingParams(defaultSort='_id')
         .errorResponse()
         .errorResponse('You are not an administrator.', 403)
     )
-    def getAssetstoreFiles(self, assetstore, limit, offset, sort, params):
-        return list(self.model('file').find(
-            query={'assetstoreId': assetstore['_id']},
-            offset=offset, limit=limit, sort=sort))
+    def getAssetstoreFiles(self, assetstore, limit, offset, sort):
+        return list(File().find(
+            query={'assetstoreId': assetstore['_id']}, offset=offset, limit=limit, sort=sort))
