@@ -26,6 +26,10 @@ from ..describe import Description, autoDescribeRoute, describeRoute
 from ..rest import Resource, RestException, filtermodel
 from ...constants import AccessType, TokenScope
 from girder.models.model_base import AccessException, GirderException
+from girder.models.assetstore import Assetstore
+from girder.models.file import File as FileModel
+from girder.models.item import Item
+from girder.models.upload import Upload
 from girder.api import access
 from girder.utility import RequestBodyStream
 from girder.utility.progress import ProgressContext
@@ -38,6 +42,8 @@ class File(Resource):
     """
     def __init__(self):
         super(File, self).__init__()
+        self._model = FileModel()
+
         self.resourceName = 'file'
         self.route('DELETE', (':id',), self.deleteFile)
         self.route('DELETE', ('upload', ':id'), self.cancelUpload)
@@ -54,19 +60,25 @@ class File(Resource):
         self.route('PUT', (':id', 'move'), self.moveFileToAssetstore)
 
     @access.public(scope=TokenScope.DATA_READ)
-    @filtermodel(model='file')
+    @filtermodel(model=FileModel)
     @autoDescribeRoute(
         Description('Get a file\'s information.')
-        .modelParam('id', model='file', level=AccessType.READ)
+        .modelParam('id', model=FileModel, level=AccessType.READ)
         .errorResponse()
         .errorResponse('Read access was denied on the file.', 403)
     )
-    def getFile(self, file, params):
+    def getFile(self, file):
         return file
 
     @access.user(scope=TokenScope.DATA_WRITE)
     @autoDescribeRoute(
         Description('Start a new upload or create an empty or link file.')
+        .notes('Use POST /file/chunk to send the contents of the file.  '
+               'The data for the first chunk of the file can be included with '
+               'this query by sending it as the body of the request using an '
+               'appropriate content-type and with the other parameters as '
+               'part of the query string.  If the entire file is uploaded via '
+               'this call, the resulting file is returned.')
         .responseClass('Upload')
         .param('parentType', 'Type being uploaded into.', enum=['folder', 'item'])
         .param('parentId', 'The ID of the parent.')
@@ -85,7 +97,7 @@ class File(Resource):
         .errorResponse('Failed to create upload.', 500)
     )
     def initUpload(self, parentType, parentId, name, size, mimeType, linkUrl, reference,
-                   assetstoreId, params):
+                   assetstoreId):
         """
         Before any bytes of the actual file are sent, a request should be made
         to initialize the upload. This creates the temporary record of the
@@ -98,8 +110,8 @@ class File(Resource):
             id=parentId, user=user, level=AccessType.WRITE, exc=True)
 
         if linkUrl is not None:
-            return self.model('file').filter(
-                self.model('file').createLinkFile(
+            return self._model.filter(
+                self._model.createLinkFile(
                     url=linkUrl, parent=parent, name=name, parentType=parentType, creator=user,
                     size=size, mimeType=mimeType), user)
         else:
@@ -108,9 +120,24 @@ class File(Resource):
             if assetstoreId:
                 self.requireAdmin(
                     user, message='You must be an admin to select a destination assetstore.')
-                assetstore = self.model('assetstore').load(assetstoreId)
+                assetstore = Assetstore().load(assetstoreId)
+
+            chunk = None
+            if size > 0 and cherrypy.request.headers.get('Content-Length'):
+                ct = cherrypy.request.body.content_type.value
+                if (ct not in cherrypy.request.body.processors and
+                        ct.split('/', 1)[0] not in cherrypy.request.body.processors):
+                    chunk = RequestBodyStream(cherrypy.request.body)
+            if chunk is not None and chunk.getSize() <= 0:
+                chunk = None
+
             try:
-                upload = self.model('upload').createUpload(
+                # TODO: This can be made more efficient by adding
+                #    save=chunk is None
+                # to the createUpload call parameters.  However, since this is
+                # a breaking change, that should be deferred until a major
+                # version upgrade.
+                upload = Upload().createUpload(
                     user=user, name=name, parentType=parentType, parent=parent, size=size,
                     mimeType=mimeType, reference=reference, assetstore=assetstore)
             except OSError as exc:
@@ -119,10 +146,12 @@ class File(Resource):
                         'Failed to create upload.', 'girder.api.v1.file.create-upload-failed')
                 raise
             if upload['size'] > 0:
+                if chunk:
+                    return Upload().handleChunk(upload, chunk, filter=True, user=user)
+
                 return upload
             else:
-                return self.model('file').filter(
-                    self.model('upload').finalizeUpload(upload), user)
+                return self._model.filter(Upload().finalizeUpload(upload), user)
 
     @access.user(scope=TokenScope.DATA_WRITE)
     @autoDescribeRoute(
@@ -130,13 +159,13 @@ class File(Resource):
         .notes('This is only required in certain non-standard upload '
                'behaviors. Clients should know which behavior models require '
                'the finalize step to be called in their behavior handlers.')
-        .modelParam('uploadId', paramType='formData')
+        .modelParam('uploadId', paramType='formData', model=Upload)
         .errorResponse(('ID was invalid.',
                         'The upload does not require finalization.',
                         'Not enough bytes have been uploaded.'))
         .errorResponse('You are not the user who initiated the upload.', 403)
     )
-    def finalizeUpload(self, upload, params):
+    def finalizeUpload(self, upload):
         user = self.getCurrentUser()
 
         if upload['userId'] != user['_id']:
@@ -150,28 +179,28 @@ class File(Resource):
                 'Server has only received %s bytes, but the file should be %s bytes.' %
                 (upload['received'], upload['size']))
 
-        file = self.model('upload').finalizeUpload(upload)
+        file = Upload().finalizeUpload(upload)
         extraKeys = file.get('additionalFinalizeKeys', ())
-        return self.model('file').filter(file, user, additionalKeys=extraKeys)
+        return self._model.filter(file, user, additionalKeys=extraKeys)
 
     @access.user(scope=TokenScope.DATA_WRITE)
     @autoDescribeRoute(
         Description('Request required offset before resuming an upload.')
-        .modelParam('uploadId', paramType='formData')
+        .modelParam('uploadId', paramType='formData', model=Upload)
         .errorResponse("The ID was invalid, or the offset did not match the server's record.")
     )
-    def requestOffset(self, upload, params):
+    def requestOffset(self, upload):
         """
         This should be called when resuming an interrupted upload. It will
         report the offset into the upload that should be used to resume.
         :param uploadId: The _id of the temp upload record being resumed.
         :returns: The offset in bytes that the client should use.
         """
-        offset = self.model('upload').requestOffset(upload)
+        offset = Upload().requestOffset(upload)
 
         if isinstance(offset, six.integer_types):
             upload['received'] = offset
-            self.model('upload').save(upload)
+            Upload().save(upload)
             return {'offset': offset}
         else:
             return offset
@@ -179,7 +208,13 @@ class File(Resource):
     @access.user(scope=TokenScope.DATA_WRITE)
     @autoDescribeRoute(
         Description('Upload a chunk of a file.')
-        .modelParam('uploadId', paramType='formData')
+        .notes('The data for the chunk should be sent as the body of the '
+               'request using an appropriate content-type and with the other '
+               'parameters as part of the query string.  Alternately, the '
+               'data can be sent as a file in the "chunk" field in multipart '
+               'form data.  Multipart uploads are much less efficient and '
+               'their use is deprecated.')
+        .modelParam('uploadId', paramType='formData', model=Upload)
         .param('offset', 'Offset of the chunk in the file.', dataType='integer',
                paramType='formData')
         .errorResponse(('ID was invalid.',
@@ -201,7 +236,7 @@ class File(Resource):
         well as passing offset and uploadId as query parameters and passing
         the chunk as the body, which is the recommended method.
 
-        Multipart uploads are @deprecated as of v2.2.0.
+        .. deprecated :: 2.2.0
         """
         if 'chunk' in params:
             chunk = params['chunk']
@@ -224,7 +259,7 @@ class File(Resource):
                 'Server has received %s bytes, but client sent offset %s.' % (
                     upload['received'], offset))
         try:
-            return self.model('upload').handleChunk(upload, chunk, filter=True, user=user)
+            return Upload().handleChunk(upload, chunk, filter=True, user=user)
         except IOError as exc:
             if exc.errno == errno.EACCES:
                 raise Exception('Failed to store upload.')
@@ -236,7 +271,7 @@ class File(Resource):
         Description('Download a file.')
         .notes('This endpoint also accepts the HTTP "Range" header for partial '
                'file downloads.')
-        .modelParam('id', model='file', level=AccessType.READ)
+        .modelParam('id', model=FileModel, level=AccessType.READ)
         .param('offset', 'Start downloading at this offset in bytes within '
                'the file.', dataType='integer', required=False, default=0)
         .param('endByte', 'If you only wish to download part of the file, '
@@ -253,7 +288,7 @@ class File(Resource):
         .errorResponse('ID was invalid.')
         .errorResponse('Read access was denied on the parent folder.', 403)
     )
-    def download(self, file, offset, endByte, contentDisposition, extraParameters, params):
+    def download(self, file, offset, endByte, contentDisposition, extraParameters):
         """
         Defers to the underlying assetstore adapter to stream a file out.
         Requires read permission on the folder that contains the file's item.
@@ -266,7 +301,7 @@ class File(Resource):
             # Currently we only support a single range.
             offset, endByte = rangeHeader[0]
 
-        return self.model('file').download(
+        return self._model.download(
             file, offset, endByte=endByte, contentDisposition=contentDisposition,
             extraParameters=extraParameters)
 
@@ -291,101 +326,101 @@ class File(Resource):
     @access.user(scope=TokenScope.DATA_WRITE)
     @autoDescribeRoute(
         Description('Delete a file by ID.')
-        .modelParam('id', model='file', level=AccessType.WRITE)
+        .modelParam('id', model=FileModel, level=AccessType.WRITE)
         .errorResponse('ID was invalid.')
         .errorResponse('Write access was denied on the parent folder.', 403)
     )
-    def deleteFile(self, file, params):
-        self.model('file').remove(file)
+    def deleteFile(self, file):
+        self._model.remove(file)
 
     @access.user(scope=TokenScope.DATA_WRITE)
     @autoDescribeRoute(
         Description('Cancel a partially completed upload.')
-        .modelParam('id', model='upload')
+        .modelParam('id', model=Upload)
         .errorResponse('ID was invalid.')
         .errorResponse('You lack permission to cancel this upload.', 403)
     )
-    def cancelUpload(self, upload, params):
+    def cancelUpload(self, upload):
         user = self.getCurrentUser()
 
         if upload['userId'] != user['_id'] and not user['admin']:
             raise AccessException('You did not initiate this upload.')
 
-        self.model('upload').cancelUpload(upload)
+        Upload().cancelUpload(upload)
         return {'message': 'Upload canceled.'}
 
     @access.user(scope=TokenScope.DATA_WRITE)
-    @filtermodel(model='file')
+    @filtermodel(model=FileModel)
     @autoDescribeRoute(
         Description('Change file metadata such as name or MIME type.')
-        .modelParam('id', model='file', level=AccessType.WRITE)
+        .modelParam('id', model=FileModel, level=AccessType.WRITE)
         .param('name', 'The name to set on the file.', required=False, strip=True)
         .param('mimeType', 'The MIME type of the file.', required=False, strip=True)
         .errorResponse('ID was invalid.')
         .errorResponse('Write access was denied on the parent folder.', 403)
     )
-    def updateFile(self, file, name, mimeType, params):
+    def updateFile(self, file, name, mimeType):
         if name is not None:
             file['name'] = name
         if mimeType is not None:
             file['mimeType'] = mimeType
 
-        return self.model('file').updateFile(file)
+        return self._model.updateFile(file)
 
     @access.user(scope=TokenScope.DATA_WRITE)
     @autoDescribeRoute(
         Description('Change the contents of an existing file.')
-        .modelParam('id', model='file', level=AccessType.WRITE)
+        .modelParam('id', model=FileModel, level=AccessType.WRITE)
         .param('size', 'Size in bytes of the new file.', dataType='integer')
         .param('reference', 'If included, this information is passed to the '
-               'data.process event when the upload is complete.',
-               required=False)
+               'data.process event when the upload is complete.', required=False)
         .param('assetstoreId', 'Direct the upload to a specific assetstore (admin-only).',
                required=False)
         .notes('After calling this, send the chunks just like you would with a '
                'normal file upload.')
     )
-    def updateFileContents(self, file, size, reference, assetstoreId, params):
+    def updateFileContents(self, file, size, reference, assetstoreId):
         user = self.getCurrentUser()
 
         assetstore = None
         if assetstoreId:
             self.requireAdmin(
                 user, message='You must be an admin to select a destination assetstore.')
-            assetstore = self.model('assetstore').load(assetstoreId)
+            assetstore = Assetstore().load(assetstoreId)
         # Create a new upload record into the existing file
-        upload = self.model('upload').createUploadToFile(
+        upload = Upload().createUploadToFile(
             file=file, user=user, size=size, reference=reference, assetstore=assetstore)
 
         if upload['size'] > 0:
             return upload
         else:
-            return self.model('file').filter(self.model('upload').finalizeUpload(upload), user)
+            return self._model.filter(Upload().finalizeUpload(upload), user)
 
     @access.admin(scope=TokenScope.DATA_WRITE)
-    @filtermodel(model='file')
+    @filtermodel(model=FileModel)
     @autoDescribeRoute(
         Description('Move a file to a different assetstore.')
-        .modelParam('id', model='file', level=AccessType.WRITE)
-        .modelParam('assetstoreId', 'The destination assetstore.', paramType='formData')
+        .modelParam('id', model=FileModel, level=AccessType.WRITE)
+        .modelParam('assetstoreId', 'The destination assetstore.', paramType='formData',
+                    model=Assetstore)
         .param('progress', 'Controls whether progress notifications will be sent.',
                dataType='boolean', default=False, required=False)
     )
-    def moveFileToAssetstore(self, file, assetstore, progress, params):
+    def moveFileToAssetstore(self, file, assetstore, progress):
         user = self.getCurrentUser()
         title = 'Moving file "%s" to assetstore "%s"' % (file['name'], assetstore['name'])
 
         with ProgressContext(progress, user=user, title=title, total=file['size']) as ctx:
-            return self.model('upload').moveFileToAssetstore(
+            return Upload().moveFileToAssetstore(
                 file=file, user=user, assetstore=assetstore, progress=ctx)
 
     @access.user(scope=TokenScope.DATA_WRITE)
-    @filtermodel(model='file')
+    @filtermodel(model=FileModel)
     @autoDescribeRoute(
         Description('Copy a file.')
-        .modelParam('id', model='file', level=AccessType.READ)
+        .modelParam('id', model=FileModel, level=AccessType.READ)
         .modelParam('itemId', description='The ID of the item to copy the file to.',
-                    level=AccessType.WRITE, paramType='formData')
+                    level=AccessType.WRITE, paramType='formData', model=Item)
     )
-    def copy(self, file, item, params):
-        return self.model('file').copyFile(file, self.getCurrentUser(), item=item)
+    def copy(self, file, item):
+        return self._model.copyFile(file, self.getCurrentUser(), item=item)
