@@ -18,10 +18,13 @@
 ###############################################################################
 
 from ..describe import Description, autoDescribeRoute
-from ..rest import Resource, RestException, filtermodel, setResponseHeader
+from ..rest import Resource, RestException, filtermodel, setResponseHeader, setContentDisposition
 from girder.utility import ziputil
 from girder.constants import AccessType, TokenScope
 from girder.api import access
+from girder.models.file import File
+from girder.models.folder import Folder
+from girder.models.item import Item as ItemModel
 
 
 class Item(Resource):
@@ -29,6 +32,8 @@ class Item(Resource):
     def __init__(self):
         super(Item, self).__init__()
         self.resourceName = 'item'
+        self._model = ItemModel()
+
         self.route('DELETE', (':id',), self.deleteItem)
         self.route('GET', (), self.find)
         self.route('GET', (':id',), self.getItem)
@@ -42,9 +47,13 @@ class Item(Resource):
         self.route('DELETE', (':id', 'metadata'), self.deleteMetadata)
 
     @access.public(scope=TokenScope.DATA_READ)
-    @filtermodel(model='item')
+    @filtermodel(model=ItemModel)
     @autoDescribeRoute(
         Description('List or search for items.')
+        .notes('You must pass either a "itemId" or "text" field'
+               'to specify how you are searching for items.  '
+               'If you omit one of these parameters the request will fail and respond : '
+               '"Invalid search mode."')
         .responseClass('Item', array=True)
         .param('folderId', 'Pass this to list all items in a folder.',
                required=False)
@@ -56,7 +65,7 @@ class Item(Resource):
         .errorResponse()
         .errorResponse('Read access was denied on the parent folder.', 403)
     )
-    def find(self, folderId, text, name, limit, offset, sort, params):
+    def find(self, folderId, text, name, limit, offset, sort):
         """
         Get a list of items with given search parameters. Currently accepted
         search modes are:
@@ -71,7 +80,7 @@ class Item(Resource):
         user = self.getCurrentUser()
 
         if folderId:
-            folder = self.model('folder').load(
+            folder = Folder().load(
                 id=folderId, user=user, level=AccessType.READ, exc=True)
             filters = {}
             if text:
@@ -81,79 +90,89 @@ class Item(Resource):
             if name:
                 filters['name'] = name
 
-            return list(self.model('folder').childItems(
+            return list(Folder().childItems(
                 folder=folder, limit=limit, offset=offset, sort=sort, filters=filters))
         elif text is not None:
-            return list(self.model('item').textSearch(
+            return list(self._model.textSearch(
                 text, user=user, limit=limit, offset=offset, sort=sort))
         else:
             raise RestException('Invalid search mode.')
 
     @access.public(scope=TokenScope.DATA_READ)
-    @filtermodel(model='item')
+    @filtermodel(model=ItemModel)
     @autoDescribeRoute(
         Description('Get an item by ID.')
         .responseClass('Item')
-        .modelParam('id', model='item', level=AccessType.READ)
+        .modelParam('id', model=ItemModel, level=AccessType.READ)
         .errorResponse('ID was invalid.')
         .errorResponse('Read access was denied for the item.', 403)
     )
-    def getItem(self, item, params):
+    def getItem(self, item):
         return item
 
     @access.user(scope=TokenScope.DATA_WRITE)
-    @filtermodel(model='item')
+    @filtermodel(model=ItemModel)
     @autoDescribeRoute(
         Description('Create a new item.')
         .responseClass('Item')
-        .modelParam('folderId', 'The ID of the parent folder.',
+        .modelParam('folderId', 'The ID of the parent folder.', model=Folder,
                     level=AccessType.WRITE, paramType='query')
         .param('name', 'Name for the item.', strip=True)
         .param('description', 'Description for the item.', required=False,
                default='', strip=True)
         .param('reuseExisting', 'Return existing item (by name) if it exists.',
                required=False, dataType='boolean', default=False)
+        .jsonParam('metadata', 'A JSON object containing the metadata keys to add',
+                   paramType='form', requireObject=True, required=False)
         .errorResponse()
         .errorResponse('Write access was denied on the parent folder.', 403)
     )
-    def createItem(self, folder, name, description, reuseExisting, params):
-        return self.model('item').createItem(
+    def createItem(self, folder, name, description, reuseExisting, metadata):
+        newItem = self._model.createItem(
             folder=folder, name=name, creator=self.getCurrentUser(), description=description,
             reuseExisting=reuseExisting)
+        if metadata:
+            newItem = self._model.setMetadata(newItem, metadata)
+        return newItem
 
     @access.user(scope=TokenScope.DATA_WRITE)
-    @filtermodel(model='item')
+    @filtermodel(model=ItemModel)
     @autoDescribeRoute(
         Description('Edit an item or move it to another folder.')
         .responseClass('Item')
-        .modelParam('id', model='item', level=AccessType.WRITE)
+        .modelParam('id', model=ItemModel, level=AccessType.WRITE)
         .param('name', 'Name for the item.', required=False, strip=True)
         .param('description', 'Description for the item.', required=False)
-        .modelParam('folderId', 'Pass this to move the item to a new folder.',
+        .modelParam('folderId', 'Pass this to move the item to a new folder.', model=Folder,
                     required=False, paramType='query', level=AccessType.WRITE)
+        .jsonParam('metadata', 'A JSON object containing the metadata keys to add',
+                   paramType='form', requireObject=True, required=False)
         .errorResponse('ID was invalid.')
         .errorResponse('Write access was denied for the item or folder.', 403)
     )
-    def updateItem(self, item, name, description, folder, params):
+    def updateItem(self, item, name, description, folder, metadata):
         if name is not None:
             item['name'] = name
         if description is not None:
             item['description'] = description
 
-        self.model('item').updateItem(item)
+        self._model.updateItem(item)
 
         if folder and folder['_id'] != item['folderId']:
-            self.model('item').move(item, folder)
+            self._model.move(item, folder)
+
+        if metadata:
+            item = self._model.setMetadata(item, metadata)
 
         return item
 
     @access.user(scope=TokenScope.DATA_WRITE)
-    @filtermodel(model='item')
+    @filtermodel(model=ItemModel)
     @autoDescribeRoute(
         Description('Set metadata fields on an item.')
         .responseClass('Item')
         .notes('Set metadata fields to null in order to delete them.')
-        .modelParam('id', model='item', level=AccessType.WRITE)
+        .modelParam('id', model=ItemModel, level=AccessType.WRITE)
         .jsonParam('metadata', 'A JSON object containing the metadata keys to add',
                    paramType='body', requireObject=True)
         .param('allowNull', 'Whether "null" is allowed as a metadata value.', required=False,
@@ -163,15 +182,15 @@ class Item(Resource):
                         'Metadata key name was invalid.'))
         .errorResponse('Write access was denied for the item.', 403)
     )
-    def setMetadata(self, item, metadata, allowNull, params):
-        return self.model('item').setMetadata(item, metadata, allowNull=allowNull)
+    def setMetadata(self, item, metadata, allowNull):
+        return self._model.setMetadata(item, metadata, allowNull=allowNull)
 
     @access.user(scope=TokenScope.DATA_WRITE)
-    @filtermodel('item')
+    @filtermodel(ItemModel)
     @autoDescribeRoute(
         Description('Delete metadata fields on an item.')
         .responseClass('Item')
-        .modelParam('id', model='item', level=AccessType.WRITE)
+        .modelParam('id', model=ItemModel, level=AccessType.WRITE)
         .jsonParam(
             'fields', 'A JSON list containing the metadata fields to delete',
             paramType='body', schema={
@@ -186,43 +205,41 @@ class Item(Resource):
                         'Metadata key name was invalid.'))
         .errorResponse('Write access was denied for the item.', 403)
     )
-    def deleteMetadata(self, item, fields, params):
-        return self.model('item').deleteMetadata(item, fields)
+    def deleteMetadata(self, item, fields):
+        return self._model.deleteMetadata(item, fields)
 
     def _downloadMultifileItem(self, item, user):
         setResponseHeader('Content-Type', 'application/zip')
-        setResponseHeader(
-            'Content-Disposition',
-            'attachment; filename="%s%s"' % (item['name'], '.zip'))
+        setContentDisposition(item['name'] + '.zip')
 
         def stream():
             zip = ziputil.ZipGenerator(item['name'])
-            for (path, file) in self.model('item').fileList(item, subpath=False):
+            for (path, file) in self._model.fileList(item, subpath=False):
                 for data in zip.addFile(file, path):
                     yield data
             yield zip.footer()
         return stream
 
     @access.public(scope=TokenScope.DATA_READ)
-    @filtermodel(model='file')
+    @filtermodel(model=File)
     @autoDescribeRoute(
         Description('Get the files within an item.')
         .responseClass('File', array=True)
-        .modelParam('id', model='item', level=AccessType.READ)
+        .modelParam('id', model=ItemModel, level=AccessType.READ)
         .pagingParams(defaultSort='name')
         .errorResponse('ID was invalid.')
         .errorResponse('Read access was denied for the item.', 403)
     )
-    def getFiles(self, item, limit, offset, sort, params):
-        return list(self.model('item').childFiles(
-            item=item, limit=limit, offset=offset, sort=sort))
+    def getFiles(self, item, limit, offset, sort):
+        return list(self._model.childFiles(item=item, limit=limit, offset=offset, sort=sort))
 
     @access.cookie
     @access.public(scope=TokenScope.DATA_READ)
     @autoDescribeRoute(
         Description('Download the contents of an item.')
-        .modelParam('id', model='item', level=AccessType.READ)
-        .param('offset', 'Byte offset into the file.', dataType='int', default=0)
+        .modelParam('id', model=ItemModel, level=AccessType.READ)
+        .param('offset', 'Byte offset into the file.', dataType='int',
+               required=False, default=0)
         .param('format', 'If unspecified, items with one file are downloaded '
                'as that file, and other items are downloaded as a zip '
                'archive.  If \'zip\', a zip archive is always sent.',
@@ -234,19 +251,20 @@ class Item(Resource):
         .param('extraParameters', 'Arbitrary data to send along with the '
                'download request, only applied for single file '
                'items.', required=False)
+        # single file items could produce other types, too.
+        .produces(['application/zip', 'application/octet-stream'])
         .errorResponse('ID was invalid.')
         .errorResponse('Read access was denied for the item.', 403)
     )
-    def download(self, item, offset, format, contentDisposition, extraParameters, params):
+    def download(self, item, offset, format, contentDisposition, extraParameters):
         user = self.getCurrentUser()
-        files = list(self.model('item').childFiles(item=item, limit=2))
+        files = list(self._model.childFiles(item=item, limit=2))
         if format not in (None, '', 'zip'):
             raise RestException('Unsupported format: %s.' % format)
         if len(files) == 1 and format != 'zip':
             if contentDisposition not in {None, 'inline', 'attachment'}:
-                raise RestException(
-                    'Unallowed contentDisposition type "%s".' % contentDisposition)
-            return self.model('file').download(
+                raise RestException('Unallowed contentDisposition type "%s".' % contentDisposition)
+            return File().download(
                 files[0], offset, contentDisposition=contentDisposition,
                 extraParameters=extraParameters)
         else:
@@ -255,33 +273,33 @@ class Item(Resource):
     @access.user(scope=TokenScope.DATA_WRITE)
     @autoDescribeRoute(
         Description('Delete an item by ID.')
-        .modelParam('id', model='item', level=AccessType.WRITE)
+        .modelParam('id', model=ItemModel, level=AccessType.WRITE)
         .errorResponse('ID was invalid.')
         .errorResponse('Write access was denied for the item.', 403)
     )
-    def deleteItem(self, item, params):
-        self.model('item').remove(item)
+    def deleteItem(self, item):
+        self._model.remove(item)
         return {'message': 'Deleted item %s.' % item['name']}
 
     @access.public(scope=TokenScope.DATA_READ)
     @autoDescribeRoute(
         Description('Get the path to the root of the item\'s hierarchy.')
-        .modelParam('id', model='item', level=AccessType.READ)
+        .modelParam('id', model=ItemModel, level=AccessType.READ)
         .errorResponse('ID was invalid.')
         .errorResponse('Read access was denied for the item.', 403)
     )
-    def rootpath(self, item, params):
-        return self.model('item').parentsToRoot(item, self.getCurrentUser())
+    def rootpath(self, item):
+        return self._model.parentsToRoot(item, self.getCurrentUser())
 
     @access.user(scope=TokenScope.DATA_WRITE)
-    @filtermodel(model='item')
+    @filtermodel(model=ItemModel)
     @autoDescribeRoute(
         Description('Copy an item.')
         .notes('If no folderId parameter is specified, creates a copy of the item in '
                'its current containing folder.')
         .responseClass('Item')
-        .modelParam('id', 'The ID of the original item.', model='item', level=AccessType.READ)
-        .modelParam('folderId', 'The ID of the parent folder.', required=False,
+        .modelParam('id', 'The ID of the original item.', model=ItemModel, level=AccessType.READ)
+        .modelParam('folderId', 'The ID of the parent folder.', required=False, model=Folder,
                     level=AccessType.WRITE)
         .param('name', 'Name for the new item.', required=False, strip=True)
         .param('description', 'Description for the new item.', required=False, strip=True)
@@ -290,11 +308,11 @@ class Item(Resource):
         .errorResponse('Read access was denied on the original item.\n\n'
                        'Write access was denied on the parent folder.', 403)
     )
-    def copyItem(self, item, folder, name, description, params):
+    def copyItem(self, item, folder, name, description):
         user = self.getCurrentUser()
 
         if folder is None:
-            folder = self.model('folder').load(
+            folder = Folder().load(
                 id=item['folderId'], user=user, level=AccessType.WRITE, exc=True)
-        return self.model('item').copyItem(
+        return self._model.copyItem(
             item, creator=user, name=name, folder=folder, description=description)
