@@ -27,9 +27,15 @@ import os
 import logging
 
 from girder.api import access
-from girder.constants import SettingKey, TokenScope, ACCESS_FLAGS, VERSION
+from girder.constants import GIRDER_ROUTE_ID, GIRDER_STATIC_ROUTE_ID, \
+    SettingKey, TokenScope, ACCESS_FLAGS, VERSION
 from girder.models.model_base import GirderException
-from girder.utility import install, plugin_utilities, system
+from girder.models.group import Group
+from girder.models.setting import Setting
+from girder.models.upload import Upload
+from girder.models.user import User
+from girder.utility import config, install, plugin_utilities, system
+from girder.utility.path import NotFoundException
 from girder.utility.progress import ProgressContext
 from ..describe import API_VERSION, Description, autoDescribeRoute
 from ..rest import Resource, RestException
@@ -47,6 +53,7 @@ class System(Resource):
         self.resourceName = 'system'
         self.route('DELETE', ('setting',), self.unsetSetting)
         self.route('GET', ('version',), self.getVersion)
+        self.route('GET', ('configuration',), self.getConfigurationOption)
         self.route('GET', ('setting',), self.getSetting)
         self.route('GET', ('plugins',), self.getPlugins)
         self.route('GET', ('access_flag',), self.getAccessFlags)
@@ -76,7 +83,7 @@ class System(Resource):
         .errorResponse('You are not a system administrator.', 403)
         .errorResponse('Failed to set system setting.', 500)
     )
-    def setSetting(self, key, value, list, params):
+    def setSetting(self, key, value, list):
         if list is None:
             list = ({'key': key, 'value': value},)
 
@@ -89,12 +96,31 @@ class System(Resource):
                     pass
 
             if value is None:
-                self.model('setting').unset(key=key)
+                Setting().unset(key=key)
             else:
-                self.model('setting').set(key=key, value=value)
+                Setting().set(key=key, value=value)
 
         return True
 
+    @access.admin(scope=TokenScope.SETTINGS_READ)
+    @autoDescribeRoute(
+        Description('Get the value of a system configuration option.')
+        .notes('Must be a system administrator to call this.')
+        .param('section', 'The section identifying the configuration option.', required=True)
+        .param('key', 'The key identifying the configuration option.', required=True)
+        .errorResponse('You are not a system administrator.', 403)
+        .errorResponse('No such option with the given section/key exists.', 404)
+    )
+    def getConfigurationOption(self, section, key, params):
+        configSection = config.getConfig().get(section)
+
+        if configSection is None:
+            raise NotFoundException('No section with that name exists.')
+        elif key not in configSection:
+            raise NotFoundException('No key with that name exists.')
+        else:
+            return configSection.get(key)
+
     @access.admin(scope=TokenScope.SETTINGS_READ)
     @autoDescribeRoute(
         Description('Get the value of a system setting, or a list of them.')
@@ -107,7 +133,7 @@ class System(Resource):
                'default value of the setting(s).', required=False)
         .errorResponse('You are not a system administrator.', 403)
     )
-    def getSetting(self, key, list, default, params):
+    def getSetting(self, key, list, default):
         getFuncName = 'get'
         funcParams = {}
         if default is not None:
@@ -117,7 +143,7 @@ class System(Resource):
                 getFuncName = 'getDefault'
             elif default:
                 raise RestException("Default was not 'none', 'default', or blank.")
-        getFunc = getattr(self.model('setting'), getFuncName)
+        getFunc = getattr(Setting(), getFuncName)
         if list is not None:
             return {k: getFunc(k, **funcParams) for k in list}
         else:
@@ -130,17 +156,21 @@ class System(Resource):
         .notes('Must be a system administrator to call this.')
         .errorResponse('You are not a system administrator.', 403)
     )
-    def getPlugins(self, params):
-        return {
+    def getPlugins(self):
+        plugins = {
             'all': plugin_utilities.findAllPlugins(),
-            'enabled': self.model('setting').get(SettingKey.PLUGINS_ENABLED)
+            'enabled': Setting().get(SettingKey.PLUGINS_ENABLED)
         }
+        failureInfo = plugin_utilities.getPluginFailureInfo()
+        if failureInfo:
+            plugins['failed'] = failureInfo
+        return plugins
 
     @access.public
     @autoDescribeRoute(
         Description('Get the version information for this server.')
     )
-    def getVersion(self, params):
+    def getVersion(self):
         version = dict(**VERSION)
         version['apiVersion'] = API_VERSION
         version['serverStartDate'] = ModuleStartTime
@@ -155,8 +185,27 @@ class System(Resource):
         .errorResponse('Required dependencies do not exist.', 500)
         .errorResponse('You are not a system administrator.', 403)
     )
-    def enablePlugins(self, plugins, params):
-        return self.model('setting').set(SettingKey.PLUGINS_ENABLED, plugins)
+    def enablePlugins(self, plugins):
+        # Determine what plugins have been disabled and remove their associated routes.
+        setting = Setting()
+        routeTable = setting.get(SettingKey.ROUTE_TABLE)
+        oldPlugins = setting.get(SettingKey.PLUGINS_ENABLED)
+        reservedRoutes = {GIRDER_ROUTE_ID, GIRDER_STATIC_ROUTE_ID}
+
+        routeTableChanged = False
+        removedRoutes = (
+            set(oldPlugins) - set(plugins) - reservedRoutes)
+
+        for route in removedRoutes:
+            if route in routeTable:
+                del routeTable[route]
+                routeTableChanged = True
+
+        if routeTableChanged:
+            setting.set(SettingKey.ROUTE_TABLE, routeTable)
+
+        # Route cleanup is done; update list of enabled plugins.
+        return setting.set(SettingKey.PLUGINS_ENABLED, plugins)
 
     @access.admin
     @autoDescribeRoute(
@@ -166,8 +215,8 @@ class System(Resource):
         .param('key', 'The key identifying the setting to unset.')
         .errorResponse('You are not a system administrator.', 403)
     )
-    def unsetSetting(self, key, params):
-        return self.model('setting').unset(key)
+    def unsetSetting(self, key):
+        return Setting().unset(key)
 
     @access.admin(scope=TokenScope.PARTIAL_UPLOAD_READ)
     @autoDescribeRoute(
@@ -190,7 +239,7 @@ class System(Resource):
         .errorResponse('You are not a system administrator.', 403)
     )
     def getPartialUploads(self, uploadId, userId, parentId, assetstoreId, minimumAge,
-                          includeUntracked, limit, offset, sort, params):
+                          includeUntracked, limit, offset, sort):
         filters = {}
         if uploadId is not None:
             filters['uploadId'] = uploadId
@@ -203,10 +252,10 @@ class System(Resource):
         if minimumAge is not None:
             filters['minimumAge'] = minimumAge
 
-        uploadList = list(self.model('upload').list(
+        uploadList = list(Upload().list(
             filters=filters, limit=limit, offset=offset, sort=sort))
         if includeUntracked and (limit == 0 or len(uploadList) < limit):
-            untrackedList = self.model('upload').untrackedUploads('list', assetstoreId)
+            untrackedList = Upload().untrackedUploads('list', assetstoreId)
             if limit == 0:
                 uploadList += untrackedList
             elif len(uploadList) < limit:
@@ -236,7 +285,7 @@ class System(Resource):
         .errorResponse('Failed to delete upload', 500)
     )
     def discardPartialUploads(self, uploadId, userId, parentId, assetstoreId,
-                              minimumAge, includeUntracked, params):
+                              minimumAge, includeUntracked):
         filters = {}
         if uploadId is not None:
             filters['uploadId'] = uploadId
@@ -248,12 +297,12 @@ class System(Resource):
             filters['parentId'] = parentId
         if minimumAge is not None:
             filters['minimumAge'] = minimumAge
-        uploadList = list(self.model('upload').list(filters=filters))
+        uploadList = list(Upload().list(filters=filters))
         # Move the results to list that isn't a cursor so we don't have to have
         # the cursor sitting around while we work on the data.
         for upload in uploadList:
             try:
-                self.model('upload').cancelUpload(upload)
+                Upload().cancelUpload(upload)
             except OSError as exc:
                 if exc.errno == errno.EACCES:
                     raise GirderException(
@@ -261,7 +310,7 @@ class System(Resource):
                         'girder.api.v1.system.delete-upload-failed')
                 raise
         if includeUntracked:
-            uploadList += self.model('upload').untrackedUploads('delete', assetstoreId)
+            uploadList += Upload().untrackedUploads('delete', assetstoreId)
         return uploadList
 
     @access.admin
@@ -270,7 +319,10 @@ class System(Resource):
         .notes('Must be a system administrator to call this.')
         .errorResponse('You are not a system administrator.', 403)
     )
-    def restartServer(self, params):
+    def restartServer(self):
+        if not config.getConfig()['server'].get('cherrypy_server', True):
+            raise RestException('Restarting of server is disabled.', 403)
+
         class Restart(cherrypy.process.plugins.Monitor):
             def __init__(self, bus, frequency=1):
                 cherrypy.process.plugins.Monitor.__init__(
@@ -302,7 +354,7 @@ class System(Resource):
                required=False, enum=('basic', 'quick', 'slow'), default='basic')
         .errorResponse('You are not a system administrator.', 403)
     )
-    def systemStatus(self, mode, params):
+    def systemStatus(self, mode):
         user = self.getCurrentUser()
         if mode != 'basic':
             self.requireAdmin(user)
@@ -312,7 +364,7 @@ class System(Resource):
 
     @access.public
     @autoDescribeRoute(Description('List all access flags available in the system.'))
-    def getAccessFlags(self, params):
+    def getAccessFlags(self):
         return ACCESS_FLAGS
 
     @access.admin
@@ -325,7 +377,7 @@ class System(Resource):
                required=False, dataType='boolean', default=False)
         .errorResponse('You are not a system administrator.', 403)
     )
-    def systemConsistencyCheck(self, progress, params):
+    def systemConsistencyCheck(self, progress):
         user = self.getCurrentUser()
         title = 'Running system consistency check'
         with ProgressContext(progress, user=user, title=title) as pc:
@@ -361,7 +413,7 @@ class System(Resource):
                required=False, default='error')
         .errorResponse('You are not a system administrator.', 403)
     )
-    def getLog(self, bytes, log, params):
+    def getLog(self, bytes, log):
         path = girder.getLogPaths()[log]
         filesize = os.path.getsize(path)
         length = int(bytes) or filesize
@@ -404,7 +456,7 @@ class System(Resource):
         .notes('Must be a system administrator to call this.')
         .errorResponse('You are not a system administrator.', 403)
     )
-    def getLogLevel(self, params):
+    def getLogLevel(self):
         level = girder.logger.getEffectiveLevel()
         return logging.getLevelName(level)
 
@@ -417,7 +469,7 @@ class System(Resource):
                default='INFO')
         .errorResponse('You are not a system administrator.', 403)
     )
-    def setLogLevel(self, level, params):
+    def setLogLevel(self, level):
         level = logging.getLevelName(level)
         for logger in (girder.logger, cherrypy.log.access_log, cherrypy.log.error_log):
             logger.setLevel(level)
@@ -433,7 +485,7 @@ class System(Resource):
         .param('dev', 'Whether to build for development mode.', required=False,
                dataType='boolean', default=False)
     )
-    def buildWebCode(self, progress, dev, params):
+    def buildWebCode(self, progress, dev):
         user = self.getCurrentUser()
 
         with ProgressContext(progress, user=user, title='Building web client code') as progress:
@@ -445,8 +497,8 @@ class System(Resource):
         .notes('Get result in the same structure as the access endpoints'
                'of collection, file, and group')
     )
-    def getCollectionCreationPolicyAccess(self, params):
-        cpp = self.model('setting').get('core.collection_create_policy')
+    def getCollectionCreationPolicyAccess(self):
+        cpp = Setting().get('core.collection_create_policy')
 
         acList = {
             'users': [{'id': x} for x in cpp.get('users', [])],
@@ -454,17 +506,23 @@ class System(Resource):
         }
 
         for user in acList['users'][:]:
-            userDoc = self.model('user').load(
+            userDoc = User().load(
                 user['id'], force=True,
                 fields=['firstName', 'lastName', 'login'])
-            user['login'] = userDoc['login']
-            user['name'] = ' '.join((userDoc['firstName'], userDoc['lastName']))
+            if userDoc is None:
+                acList['users'].remove(user)
+            else:
+                user['login'] = userDoc['login']
+                user['name'] = ' '.join((userDoc['firstName'], userDoc['lastName']))
 
         for grp in acList['groups'][:]:
-            grpDoc = self.model('group').load(
+            grpDoc = Group().load(
                 grp['id'], force=True, fields=['name', 'description'])
-            grp['name'] = grpDoc['name']
-            grp['description'] = grpDoc['description']
+            if grpDoc is None:
+                acList['groups'].remove(grp)
+            else:
+                grp['name'] = grpDoc['name']
+                grp['description'] = grpDoc['description']
 
         return acList
 
