@@ -25,7 +25,10 @@ from ..describe import Description, autoDescribeRoute
 from girder.api import access
 from girder.api.rest import Resource, RestException, AccessException, filtermodel, setCurrentUser
 from girder.constants import AccessType, SettingKey, TokenScope
-from girder.models.token import genToken
+from girder.models.password import Password
+from girder.models.setting import Setting
+from girder.models.token import genToken, Token
+from girder.models.user import User as UserModel
 from girder.utility import mail_utils
 
 
@@ -35,6 +38,7 @@ class User(Resource):
     def __init__(self):
         super(User, self).__init__()
         self.resourceName = 'user'
+        self._model = UserModel()
 
         self.route('DELETE', ('authentication',), self.logout)
         self.route('DELETE', (':id',), self.deleteUser)
@@ -43,6 +47,7 @@ class User(Resource):
         self.route('GET', ('authentication',), self.login)
         self.route('GET', (':id',), self.getUser)
         self.route('GET', (':id', 'details'), self.getUserDetails)
+        self.route('GET', ('details',), self.getUsersDetails)
         self.route('POST', (), self.createUser)
         self.route('PUT', (':id',), self.updateUser)
         self.route('PUT', ('password',), self.changePassword)
@@ -56,36 +61,36 @@ class User(Resource):
         self.route('POST', ('verification',), self.sendVerificationEmail)
 
     @access.public
-    @filtermodel(model='user')
+    @filtermodel(model=UserModel)
     @autoDescribeRoute(
         Description('List or search for users.')
         .responseClass('User', array=True)
         .param('text', "Pass this to perform a full text search for items.", required=False)
         .pagingParams(defaultSort='lastName')
     )
-    def find(self, text, limit, offset, sort, params):
-        return list(self.model('user').search(
+    def find(self, text, limit, offset, sort):
+        return list(self._model.search(
             text=text, user=self.getCurrentUser(), offset=offset, limit=limit, sort=sort))
 
     @access.public(scope=TokenScope.USER_INFO_READ)
-    @filtermodel(model='user')
+    @filtermodel(model=UserModel)
     @autoDescribeRoute(
         Description('Get a user by ID.')
         .responseClass('User')
-        .modelParam('id', model='user', level=AccessType.READ)
+        .modelParam('id', model=UserModel, level=AccessType.READ)
         .errorResponse('ID was invalid.')
         .errorResponse('You do not have permission to see this user.', 403)
     )
-    def getUser(self, user, params):
+    def getUser(self, user):
         return user
 
     @access.public(scope=TokenScope.USER_INFO_READ)
-    @filtermodel(model='user')
+    @filtermodel(model=UserModel)
     @autoDescribeRoute(
         Description('Retrieve the currently logged-in user information.')
         .responseClass('User')
     )
-    def getMe(self, params):
+    def getMe(self):
         return self.getCurrentUser()
 
     @access.public
@@ -96,7 +101,7 @@ class User(Resource):
         .errorResponse('Missing Authorization header.', 401)
         .errorResponse('Invalid login or password.', 403)
     )
-    def login(self, params):
+    def login(self):
         user, token = self.getCurrentUser(returnToken=True)
 
         # Only create and send new cookie if user isn't already sending a valid one.
@@ -117,13 +122,13 @@ class User(Resource):
                 raise RestException('Invalid HTTP Authorization header', 401)
 
             login, password = credentials.split(':', 1)
-            user = self.model('user').authenticate(login, password)
+            user = self._model.authenticate(login, password)
 
             setCurrentUser(user)
             token = self.sendAuthTokenCookie(user)
 
         return {
-            'user': self.model('user').filter(user, user),
+            'user': self._model.filter(user, user),
             'authToken': {
                 'token': token['_id'],
                 'expires': token['expires'],
@@ -138,15 +143,15 @@ class User(Resource):
         .responseClass('Token')
         .notes('Attempts to delete your authentication cookie.')
     )
-    def logout(self, params):
+    def logout(self):
         token = self.getCurrentToken()
         if token:
-            self.model('token').remove(token)
+            Token().remove(token)
         self.deleteAuthTokenCookie()
         return {'message': 'Logged out.'}
 
     @access.public
-    @filtermodel(model='user', addFields={'authToken'})
+    @filtermodel(model=UserModel, addFields={'authToken'})
     @autoDescribeRoute(
         Description('Create a new user.')
         .responseClass('User')
@@ -160,10 +165,10 @@ class User(Resource):
         .errorResponse('A parameter was invalid, or the specified login or'
                        ' email already exists in the system.')
     )
-    def createUser(self, login, email, firstName, lastName, password, admin, params):
+    def createUser(self, login, email, firstName, lastName, password, admin):
         currentUser = self.getCurrentUser()
 
-        regPolicy = self.model('setting').get(SettingKey.REGISTRATION_POLICY)
+        regPolicy = Setting().get(SettingKey.REGISTRATION_POLICY)
 
         if not currentUser or not currentUser['admin']:
             admin = False
@@ -172,11 +177,11 @@ class User(Resource):
                     'Registration on this instance is closed. Contact an '
                     'administrator to create an account for you.')
 
-        user = self.model('user').createUser(
+        user = self._model.createUser(
             login=login, password=password, email=email, firstName=firstName,
             lastName=lastName, admin=admin)
 
-        if not currentUser and self.model('user').canLogin(user):
+        if not currentUser and self._model.canLogin(user):
             setCurrentUser(user)
             token = self.sendAuthTokenCookie(user)
             user['authToken'] = {
@@ -188,19 +193,28 @@ class User(Resource):
     @access.user
     @autoDescribeRoute(
         Description('Delete a user by ID.')
-        .modelParam('id', model='user', level=AccessType.ADMIN)
+        .modelParam('id', model=UserModel, level=AccessType.ADMIN)
         .errorResponse('ID was invalid.')
         .errorResponse('You do not have permission to delete this user.', 403)
     )
-    def deleteUser(self, user, params):
-        self.model('user').remove(user)
+    def deleteUser(self, user):
+        self._model.remove(user)
         return {'message': 'Deleted user %s.' % user['login']}
 
+    @access.admin
+    @autoDescribeRoute(
+        Description('Get detailed information about all users.')
+        .errorResponse('You are not a system administrator.', 403)
+    )
+    def getUsersDetails(self):
+        nUsers = self._model.find().count()
+        return {'nUsers': nUsers}
+
     @access.user
-    @filtermodel(model='user')
+    @filtermodel(model=UserModel)
     @autoDescribeRoute(
         Description("Update a user's information.")
-        .modelParam('id', model='user', level=AccessType.WRITE)
+        .modelParam('id', model=UserModel, level=AccessType.WRITE)
         .param('firstName', 'First name of the user.')
         .param('lastName', 'Last name of the user.')
         .param('email', 'The email of the user.')
@@ -212,7 +226,7 @@ class User(Resource):
         .errorResponse(('You do not have write access for this user.',
                         'Must be an admin to create an admin.'), 403)
     )
-    def updateUser(self, user, firstName, lastName, email, admin, status, params):
+    def updateUser(self, user, firstName, lastName, email, admin, status):
         user['firstName'] = firstName
         user['lastName'] = lastName
         user['email'] = email
@@ -230,22 +244,22 @@ class User(Resource):
                 raise AccessException('Only admins may change status.')
             if user['status'] == 'pending' and status == 'enabled':
                 # Send email on the 'pending' -> 'enabled' transition
-                self.model('user')._sendApprovedEmail(user)
+                self._model._sendApprovedEmail(user)
             user['status'] = status
 
-        return self.model('user').save(user)
+        return self._model.save(user)
 
     @access.admin
     @autoDescribeRoute(
         Description('Change a user\'s password.')
         .notes('Only administrators may use this endpoint.')
-        .modelParam('id', model='user', level=AccessType.ADMIN)
+        .modelParam('id', model=UserModel, level=AccessType.ADMIN)
         .param('password', 'The user\'s new password.')
         .errorResponse('You are not an administrator.', 403)
         .errorResponse('The new password is invalid.')
     )
-    def changeUserPassword(self, user, password, params):
-        self.model('user').setPassword(user, password)
+    def changeUserPassword(self, user, password):
+        self._model.setPassword(user, password)
         return {'message': 'Password changed.'}
 
     @access.user
@@ -257,28 +271,26 @@ class User(Resource):
                         'Your old password is incorrect.'), 401)
         .errorResponse('Your new password is invalid.')
     )
-    def changePassword(self, old, new, params):
+    def changePassword(self, old, new):
         user = self.getCurrentUser()
         token = None
 
         if not old:
             raise RestException('Old password must not be empty.')
 
-        if (not self.model('password').hasPassword(user) or
-                not self.model('password').authenticate(user, old)):
+        if not Password().hasPassword(user) or not Password().authenticate(user, old):
             # If not the user's actual password, check for temp access token
-            token = self.model('token').load(old, force=True, objectId=False, exc=False)
+            token = Token().load(old, force=True, objectId=False, exc=False)
             if (not token or not token.get('userId') or
                     token['userId'] != user['_id'] or
-                    not self.model('token').hasScope(
-                        token, TokenScope.TEMPORARY_USER_AUTH)):
+                    not Token().hasScope(token, TokenScope.TEMPORARY_USER_AUTH)):
                 raise AccessException('Old password is incorrect.')
 
-        self.model('user').setPassword(user, new)
+        self._model.setPassword(user, new)
 
         if token:
             # Remove the temporary access token if one was used
-            self.model('token').remove(token)
+            Token().remove(token)
 
         return {'message': 'Password changed.'}
 
@@ -288,8 +300,8 @@ class User(Resource):
         .param('email', 'Your email address.', strip=True)
         .errorResponse('That email does not exist in the system.')
     )
-    def resetPassword(self, email, params):
-        user = self.model('user').findOne({'email': email.lower()})
+    def resetPassword(self, email):
+        user = self._model.findOne({'email': email.lower()})
         if user is None:
             raise RestException('That email is not registered.')
 
@@ -298,8 +310,10 @@ class User(Resource):
         html = mail_utils.renderTemplate('resetPassword.mako', {
             'password': randomPass
         })
-        mail_utils.sendEmail(to=email, subject='Girder: Password reset', text=html)
-        self.model('user').setPassword(user, randomPass)
+        mail_utils.sendEmail(
+            to=email, subject='%s: Password reset' % Setting().get(SettingKey.BRAND_NAME),
+            text=html)
+        self._model.setPassword(user, randomPass)
         return {'message': 'Sent password reset email.'}
 
     @access.public
@@ -309,14 +323,13 @@ class User(Resource):
         .param('email', 'Your email address.', strip=True)
         .errorResponse('That email does not exist in the system.')
     )
-    def generateTemporaryPassword(self, email, params):
-        user = self.model('user').findOne({'email': email.lower()})
+    def generateTemporaryPassword(self, email):
+        user = self._model.findOne({'email': email.lower()})
 
         if not user:
             raise RestException('That email is not registered.')
 
-        token = self.model('token').createToken(
-            user, days=1, scope=TokenScope.TEMPORARY_USER_AUTH)
+        token = Token().createToken(user, days=1, scope=TokenScope.TEMPORARY_USER_AUTH)
 
         url = '%s#useraccount/%s/token/%s' % (
             mail_utils.getEmailUrlPrefix(), str(user['_id']), str(token['_id']))
@@ -325,7 +338,10 @@ class User(Resource):
             'url': url,
             'token': str(token['_id'])
         })
-        mail_utils.sendEmail(to=email, subject='Girder: Temporary access', text=html)
+        mail_utils.sendEmail(
+            to=email, subject='%s: Temporary access' % Setting().get(SettingKey.BRAND_NAME),
+            text=html
+        )
         return {'message': 'Sent temporary access email.'}
 
     @access.public
@@ -333,15 +349,15 @@ class User(Resource):
         Description('Check if a specified token is a temporary access token '
                     'for the specified user.  If the token is valid, returns '
                     'information on the token and user.')
-        .modelParam('id', 'The user ID to check.', model='user', force=True)
+        .modelParam('id', 'The user ID to check.', model=UserModel, force=True)
         .param('token', 'The token to check.')
         .errorResponse('The token does not grant temporary access to the specified user.', 401)
     )
-    def checkTemporaryPassword(self, user, token, params):
-        token = self.model('token').load(
+    def checkTemporaryPassword(self, user, token):
+        token = Token().load(
             token, user=user, level=AccessType.ADMIN, objectId=False, exc=True)
         delta = (token['expires'] - datetime.datetime.utcnow()).total_seconds()
-        hasScope = self.model('token').hasScope(token, TokenScope.TEMPORARY_USER_AUTH)
+        hasScope = Token().hasScope(token, TokenScope.TEMPORARY_USER_AUTH)
 
         if token.get('userId') != user['_id'] or delta <= 0 or not hasScope:
             raise AccessException('The token does not grant temporary access to this user.')
@@ -352,7 +368,7 @@ class User(Resource):
         authToken = self.sendAuthTokenCookie(user)
 
         return {
-            'user': self.model('user').filter(user, user),
+            'user': self._model.filter(user, user),
             'authToken': {
                 'token': authToken['_id'],
                 'expires': authToken['expires'],
@@ -364,41 +380,41 @@ class User(Resource):
     @access.public
     @autoDescribeRoute(
         Description('Get detailed information about a user.')
-        .modelParam('id', model='user', level=AccessType.READ)
+        .modelParam('id', model=UserModel, level=AccessType.READ)
         .errorResponse()
         .errorResponse('Read access was denied on the user.', 403)
     )
-    def getUserDetails(self, user, params):
+    def getUserDetails(self, user):
         return {
-            'nFolders': self.model('user').countFolders(
+            'nFolders': self._model.countFolders(
                 user, filterUser=self.getCurrentUser(), level=AccessType.READ)
         }
 
     @access.public
     @autoDescribeRoute(
         Description('Verify an email address using a token.')
-        .modelParam('id', 'The user ID to check.', model='user', force=True)
+        .modelParam('id', 'The user ID to check.', model=UserModel, force=True)
         .param('token', 'The token to check.')
         .errorResponse('The token is invalid or expired.', 401)
     )
-    def verifyEmail(self, user, token, params):
-        token = self.model('token').load(
+    def verifyEmail(self, user, token):
+        token = Token().load(
             token, user=user, level=AccessType.ADMIN, objectId=False, exc=True)
         delta = (token['expires'] - datetime.datetime.utcnow()).total_seconds()
-        hasScope = self.model('token').hasScope(token, TokenScope.EMAIL_VERIFICATION)
+        hasScope = Token().hasScope(token, TokenScope.EMAIL_VERIFICATION)
 
         if token.get('userId') != user['_id'] or delta <= 0 or not hasScope:
             raise AccessException('The token is invalid or expired.')
 
         user['emailVerified'] = True
-        self.model('token').remove(token)
-        user = self.model('user').save(user)
+        Token().remove(token)
+        user = self._model.save(user)
 
-        if self.model('user').canLogin(user):
+        if self._model.canLogin(user):
             setCurrentUser(user)
             authToken = self.sendAuthTokenCookie(user)
             return {
-                'user': self.model('user').filter(user, user),
+                'user': self._model.filter(user, user),
                 'authToken': {
                     'token': authToken['_id'],
                     'expires': authToken['expires'],
@@ -408,7 +424,7 @@ class User(Resource):
             }
         else:
             return {
-                'user': self.model('user').filter(user, user),
+                'user': self._model.filter(user, user),
                 'message': 'Email verification succeeded.'
             }
 
@@ -418,12 +434,12 @@ class User(Resource):
         .param('login', 'Your login or email address.', strip=True)
         .errorResponse('That login is not registered.', 401)
     )
-    def sendVerificationEmail(self, login, params):
+    def sendVerificationEmail(self, login):
         loginField = 'email' if '@' in login else 'login'
-        user = self.model('user').findOne({loginField: login.lower()})
+        user = self._model.findOne({loginField: login.lower()})
 
         if not user:
             raise RestException('That login is not registered.', 401)
 
-        self.model('user')._sendVerificationEmail(user)
+        self._model._sendVerificationEmail(user)
         return {'message': 'Sent verification email.'}
