@@ -34,7 +34,7 @@ var Collection = Backbone.Collection.extend({
     append: false,
 
     /**
-     * Client-side filtering: set to a function that takes an instace of this
+     * Client-side filtering: set to a function that takes an instance of this
      * collection's model and returns true for each instance that should be
      * included in this collection.  For the sake of least surprise, this
      * property should not be changed in the definition of collections, but
@@ -129,8 +129,7 @@ var Collection = Backbone.Collection.extend({
      */
     fetch: function (params, reset) {
         if (this.altUrl === null && this.resourceName === null) {
-            alert('Error: You must set an altUrl or resourceName on your collection.');
-            return;
+            throw new Error('An altUrl or resourceName must be set on the Collection.');
         }
 
         if (this.filterFunc && !this.append) {
@@ -156,7 +155,7 @@ var Collection = Backbone.Collection.extend({
 
         function fetchListFragment() {
             var xhr = restRequest({
-                path: this.altUrl || this.resourceName,
+                url: this.altUrl || this.resourceName,
                 data: _.extend({
                     limit: limit,
                     offset: this.offset,
@@ -165,7 +164,7 @@ var Collection = Backbone.Collection.extend({
                 }, this.params)
             });
 
-            var result = $.when(xhr).then((list) => {
+            var result = xhr.then((list) => {
                 if (this.pageLimit > 0 && list.length > this.pageLimit) {
                     // This means we have more pages to display still. Pop off
                     // the extra that we fetched.
@@ -249,12 +248,13 @@ var Collection = Backbone.Collection.extend({
                         if (this.append && !reset) {
                             this.add(finalList);
                         } else {
-                            this.reset(finalList, this.offset);
+                            this.reset(finalList);
                         }
                     }
 
                     this.trigger('g:changed');
                 }
+                return undefined;
             });
             xhr.girder = {fetch: true};
             return result;
