@@ -1,3 +1,4 @@
+import $ from 'jquery';
 import _ from 'underscore';
 
 import Model from 'girder/models/Model';
@@ -20,13 +21,12 @@ var AccessControlledModel = Model.extend({
      */
     updateAccess: function (params) {
         if (this.altUrl === null && this.resourceName === null) {
-            alert('Error: You must set an altUrl or a resourceName on your model.');
-            return;
+            throw new Error('An altUrl or resourceName must be set on the Model.');
         }
 
         return restRequest({
-            path: (this.altUrl || this.resourceName) + '/' + this.get('_id') + '/access',
-            type: 'PUT',
+            url: `${this.altUrl || this.resourceName}/${this.id}/access`,
+            method: 'PUT',
             data: _.extend({
                 access: JSON.stringify(this.get('access')),
                 public: this.get('public'),
@@ -34,7 +34,7 @@ var AccessControlledModel = Model.extend({
             }, params || {})
         }).done(_.bind(function () {
             this.trigger('g:accessListSaved');
-        }, this)).error(_.bind(function (err) {
+        }, this)).fail(_.bind(function (err) {
             this.trigger('g:error', err);
         }, this));
     },
@@ -48,14 +48,13 @@ var AccessControlledModel = Model.extend({
      */
     fetchAccess: function (force) {
         if (this.altUrl === null && this.resourceName === null) {
-            alert('Error: You must set an altUrl or a resourceName on your model.');
-            return;
+            throw new Error('An altUrl or resourceName must be set on the Model.');
         }
 
         if (!this.get('access') || force) {
             return restRequest({
-                path: (this.altUrl || this.resourceName) + '/' + this.get('_id') + '/access',
-                type: 'GET'
+                url: `${this.altUrl || this.resourceName}/${this.id}/access`,
+                method: 'GET'
             }).done(_.bind(function (resp) {
                 if (resp.access) {
                     this.set(resp);
@@ -64,12 +63,12 @@ var AccessControlledModel = Model.extend({
                 }
                 this.trigger('g:accessFetched');
                 return resp;
-            }, this)).error(_.bind(function (err) {
+            }, this)).fail(_.bind(function (err) {
                 this.trigger('g:error', err);
             }, this));
         } else {
             this.trigger('g:accessFetched');
-            return $.when(this.get('access'));
+            return $.Deferred().resolve(this.get('access')).promise();
         }
     }
 });
