@@ -8,7 +8,7 @@ var ApiKeyModel = AccessControlledModel.extend({
 
     setActive: function (active) {
         return restRequest({
-            path: 'api_key/' + this.id,
+            url: `api_key/${this.id}`,
             method: 'PUT',
             data: {
                 active: active
@@ -22,10 +22,10 @@ var ApiKeyModel = AccessControlledModel.extend({
     save: function () {
         // Scope needs to be sent to the server as JSON
         var scope = this.get('scope');
-        this.attributes.scope = JSON.stringify(scope);
+        this.set('scope', JSON.stringify(scope), {silent: true});  // eslint-disable-line backbone/no-silent
         var promise = AccessControlledModel.prototype.save.call(this, arguments);
         // Restore scope to its original state
-        this.attributes.scope = scope;
+        this.set('scope', scope, {silent: true});  // eslint-disable-line backbone/no-silent
         return promise;
     }
 });
