@@ -19,12 +19,12 @@ var GroupModel = AccessControlledModel.extend({
     sendInvitation: function (userId, accessType, request, params) {
         params = params || {};
         return restRequest({
-            path: this.resourceName + '/' + this.get('_id') + '/invitation',
+            url: `${this.resourceName}/${this.id}/invitation`,
             data: _.extend({
                 userId: userId,
                 level: accessType
             }, params),
-            type: 'POST',
+            method: 'POST',
             error: null
         }).done(_.bind(function (resp) {
             this.set(resp);
@@ -37,7 +37,7 @@ var GroupModel = AccessControlledModel.extend({
                 }
             }
             this.trigger('g:invited');
-        }, this)).error(_.bind(function (err) {
+        }, this)).fail(_.bind(function (err) {
             this.trigger('g:error', err);
         }, this));
     },
@@ -48,8 +48,8 @@ var GroupModel = AccessControlledModel.extend({
      */
     joinGroup: function () {
         return restRequest({
-            path: this.resourceName + '/' + this.get('_id') + '/member',
-            type: 'POST'
+            url: `${this.resourceName}/${this.id}/member`,
+            method: 'POST'
         }).done(_.bind(function (resp) {
             getCurrentUser().addToGroup(this.get('_id'));
             getCurrentUser().removeInvitation(this.get('_id'));
@@ -57,7 +57,7 @@ var GroupModel = AccessControlledModel.extend({
             this.set(resp);
 
             this.trigger('g:joined');
-        }, this)).error(_.bind(function (err) {
+        }, this)).fail(_.bind(function (err) {
             this.trigger('g:error', err);
         }, this));
     },
@@ -69,13 +69,13 @@ var GroupModel = AccessControlledModel.extend({
      */
     requestInvitation: function () {
         return restRequest({
-            path: this.resourceName + '/' + this.get('_id') + '/member',
-            type: 'POST'
+            url: `${this.resourceName}/${this.id}/member`,
+            method: 'POST'
         }).done(_.bind(function (resp) {
             this.set(resp);
 
             this.trigger('g:inviteRequested');
-        }, this)).error(_.bind(function (err) {
+        }, this)).fail(_.bind(function (err) {
             this.trigger('g:error', err);
         }, this));
     },
@@ -94,16 +94,16 @@ var GroupModel = AccessControlledModel.extend({
             role = 'admin';
         }
         return restRequest({
-            path: this.resourceName + '/' + this.get('_id') + '/' + role,
+            url: `${this.resourceName}/${this.id}/${role}`,
             data: {
                 userId: user.get('_id')
             },
-            type: 'POST'
+            method: 'POST'
         }).done(_.bind(function (resp) {
             this.set(resp);
 
             this.trigger('g:promoted');
-        }, this)).error(_.bind(function (err) {
+        }, this)).fail(_.bind(function (err) {
             this.trigger('g:error', err);
         }, this));
     },
@@ -122,14 +122,13 @@ var GroupModel = AccessControlledModel.extend({
             role = 'admin';
         }
         return restRequest({
-            path: this.resourceName + '/' + this.get('_id') + '/' + role +
-                '?userId=' + userId,
-            type: 'DELETE'
+            url: `${this.resourceName}/${this.id}/${role}?userId=${userId}`,
+            method: 'DELETE'
         }).done(_.bind(function (resp) {
             this.set(resp);
 
             this.trigger('g:demoted');
-        }, this)).error(_.bind(function (err) {
+        }, this)).fail(_.bind(function (err) {
             this.trigger('g:error', err);
         }, this));
     },
@@ -142,9 +141,8 @@ var GroupModel = AccessControlledModel.extend({
      */
     removeMember: function (userId) {
         return restRequest({
-            path: this.resourceName + '/' + this.get('_id') +
-                  '/member?userId=' + userId,
-            type: 'DELETE'
+            url: `${this.resourceName}/${this.id}/member?userId=${userId}`,
+            method: 'DELETE'
         }).done(_.bind(function (resp) {
             if (userId === getCurrentUser().get('_id')) {
                 getCurrentUser().removeFromGroup(this.get('_id'));
@@ -153,7 +151,7 @@ var GroupModel = AccessControlledModel.extend({
             this.set(resp);
 
             this.trigger('g:removed');
-        }, this)).error(_.bind(function (err) {
+        }, this)).fail(_.bind(function (err) {
             this.trigger('g:error', err);
         }, this));
     },
@@ -197,4 +195,3 @@ var GroupModel = AccessControlledModel.extend({
 });
 
 export default GroupModel;
-
