@@ -6,18 +6,18 @@ var MetadataMixin = {
     _sendMetadata: function (metadata, successCallback, errorCallback, opts) {
         opts = opts || {};
         restRequest({
-            path: opts.path ||
+            url: opts.path ||
                 ((this.altUrl || this.resourceName) + `/${this.id}/metadata?allowNull=true`),
             contentType: 'application/json',
             data: JSON.stringify(metadata),
-            type: 'PUT',
+            method: 'PUT',
             error: null
         }).done(_.bind(function (resp) {
             this.set(opts.field || 'meta', resp.meta);
             if (_.isFunction(successCallback)) {
                 successCallback();
             }
-        }, this)).error(_.bind(function (err) {
+        }, this)).fail(_.bind(function (err) {
             err.message = err.responseJSON.message;
             if (_.isFunction(errorCallback)) {
                 errorCallback(err);
@@ -44,18 +44,18 @@ var MetadataMixin = {
             key = [key];
         }
         restRequest({
-            path: opts.path ||
+            url: opts.path ||
                 ((this.altUrl || this.resourceName) + `/${this.id}/metadata`),
             contentType: 'application/json',
             data: JSON.stringify(key),
-            type: 'DELETE',
+            method: 'DELETE',
             error: null
-        }).done(resp => {
+        }).done((resp) => {
             this.set(opts.field || 'meta', resp.meta);
             if (_.isFunction(successCallback)) {
                 successCallback();
             }
-        }).error(err => {
+        }).fail((err) => {
             err.message = err.responseJSON.message;
             if (_.isFunction(errorCallback)) {
                 errorCallback(err);
@@ -86,4 +86,3 @@ var MetadataMixin = {
 };
 
 export default MetadataMixin;
-
