@@ -21,7 +21,7 @@ var UserModel = Model.extend({
      */
     current: function () {
         fetchCurrentUser()
-            .then(_.bind(function (user) {
+            .done(_.bind(function (user) {
                 if (user) {
                     this.set(user);
                 } else {
@@ -88,16 +88,16 @@ var UserModel = Model.extend({
      */
     changePassword: function (oldPassword, newPassword) {
         return restRequest({
-            path: this.resourceName + '/password',
+            url: `${this.resourceName}/password`,
             data: {
                 old: oldPassword,
                 new: newPassword
             },
-            type: 'PUT',
+            method: 'PUT',
             error: null
         }).done(_.bind(function () {
             this.trigger('g:passwordChanged');
-        }, this)).error(_.bind(function (err) {
+        }, this)).fail(_.bind(function (err) {
             this.trigger('g:error', err);
         }, this));
     },
@@ -107,19 +107,18 @@ var UserModel = Model.extend({
      */
     adminChangePassword: function (newPassword) {
         return restRequest({
-            path: this.resourceName + '/' + this.id + '/password',
+            url: `${this.resourceName}/${this.id}/password`,
             data: {
                 password: newPassword
             },
-            type: 'PUT',
+            method: 'PUT',
             error: null
         }).done(_.bind(function () {
             this.trigger('g:passwordChanged');
-        }, this)).error(_.bind(function (err) {
+        }, this)).fail(_.bind(function (err) {
             this.trigger('g:error', err);
         }, this));
     }
 });
 
 export default UserModel;
-
