@@ -1,6 +1,10 @@
 import $ from 'jquery';
 import _ from 'underscore';
 import Backbone from 'backbone';
+import 'bootstrap/dist/css/bootstrap.css';
+import 'bootstrap/js/alert';
+
+import 'girder/utilities/jquery/girderModal';
 
 import events from 'girder/events';
 import eventStream from 'girder/utilities/EventStream';
@@ -26,11 +30,6 @@ import 'girder/routes';
 import 'girder/stylesheets/layout/global.styl';
 import 'girder/stylesheets/layout/layout.styl';
 
-import 'girder/utilities/jquery/girderModal';
-
-import 'bootstrap/dist/css/bootstrap.css';
-import 'bootstrap/js/alert';
-
 var App = View.extend({
     /**
      * @param {object} [settings]
@@ -39,6 +38,10 @@ var App = View.extend({
     initialize: function (settings) {
         this._started = false;
         settings = settings || {};
+        this.contactEmail = settings.contactEmail || null;
+        this.brandName = settings.brandName || null;
+        this.bannerColor = settings.bannerColor || null;
+
         if (settings.start === undefined || settings.start) {
             this.start();
         }
@@ -123,7 +126,9 @@ var App = View.extend({
      */
     _createLayout: function () {
         this.headerView = new LayoutHeaderView({
-            parentView: this
+            parentView: this,
+            brandName: this.brandName,
+            bannerColor: this.bannerColor
         });
 
         this.globalNavView = new LayoutGlobalNavView({
@@ -131,7 +136,8 @@ var App = View.extend({
         });
 
         this.footerView = new LayoutFooterView({
-            parentView: this
+            parentView: this,
+            contactEmail: this.contactEmail
         });
 
         this.progressListView = new ProgressListView({
@@ -219,7 +225,8 @@ var App = View.extend({
 
             settings = _.extend(settings, {
                 el: this.$('#g-app-body-container'),
-                parentView: this
+                parentView: this,
+                brandName: this.brandName
             });
 
             /* We let the view be created in this way even though it is
