@@ -1,3 +1,4 @@
+import $ from 'jquery';
 import _ from 'underscore';
 
 import AccessWidget from 'girder/views/widgets/AccessWidget';
@@ -12,7 +13,7 @@ import SystemConfigurationTemplate from 'girder/templates/body/systemConfigurati
 import 'girder/stylesheets/body/systemConfig.styl';
 
 import 'bootstrap/js/collapse';
-import 'bootstrap/js/tooltip';
+import 'bootstrap/js/transition';
 import 'bootstrap-switch'; // /dist/js/bootstrap-switch.js',
 import 'bootstrap-switch/dist/css/bootstrap3/bootstrap-switch.css';
 
@@ -23,7 +24,7 @@ var SystemConfigurationView = View.extend({
     events: {
         'submit .g-settings-form': function (event) {
             event.preventDefault();
-            this.$('.g-submit-settings').addClass('disabled');
+            this.$('.g-submit-settings').girderEnable(false);
             this.$('#g-settings-error-message').empty();
 
             this.$('#g-core-collection-create-policy').val(JSON.stringify(this._covertCollectionCreationPolicy()));
@@ -32,7 +33,7 @@ var SystemConfigurationView = View.extend({
                     return {
                         key: key,
                         value: _.object(_.map($('.g-core-route-table'), function (el) {
-                            return [$(el).data('webroot-name'), $(el).val()];
+                            return [$(el).data('webrootName'), $(el).val()];
                         }))
                     };
                 }
@@ -44,32 +45,38 @@ var SystemConfigurationView = View.extend({
             }, this);
 
             restRequest({
-                type: 'PUT',
-                path: 'system/setting',
+                method: 'PUT',
+                url: 'system/setting',
                 data: {
                     list: JSON.stringify(settings)
                 },
                 error: null
             }).done(_.bind(function () {
-                this.$('.g-submit-settings').removeClass('disabled');
+                this.$('.g-submit-settings').girderEnable(true);
                 events.trigger('g:alert', {
                     icon: 'ok',
                     text: 'Settings saved.',
                     type: 'success',
                     timeout: 4000
                 });
-            }, this)).error(_.bind(function (resp) {
-                this.$('.g-submit-settings').removeClass('disabled');
+            }, this)).fail(_.bind(function (resp) {
+                this.$('.g-submit-settings').girderEnable(true);
                 this.$('#g-settings-error-message').text(resp.responseJSON.message);
             }, this));
         },
-        'click #g-restart-server': restartServerPrompt
+        'click #g-restart-server': restartServerPrompt,
+        'click #g-core-banner-default-color': function () {
+            this.$('#g-core-banner-color').val(this.defaults['core.banner_color']);
+        }
     },
 
     initialize: function () {
         cancelRestRequests('fetch');
 
         var keys = [
+            'core.contact_email_address',
+            'core.brand_name',
+            'core.banner_color',
             'core.cookie_lifetime',
             'core.email_from_address',
             'core.email_host',
@@ -92,8 +99,8 @@ var SystemConfigurationView = View.extend({
         ];
         this.settingsKeys = keys;
         restRequest({
-            path: 'system/setting',
-            type: 'GET',
+            url: 'system/setting',
+            method: 'GET',
             data: {
                 list: JSON.stringify(keys),
                 default: 'none'
@@ -101,8 +108,8 @@ var SystemConfigurationView = View.extend({
         }).done(_.bind(function (resp) {
             this.settings = resp;
             restRequest({
-                path: 'system/setting',
-                type: 'GET',
+                url: 'system/setting',
+                method: 'GET',
                 data: {
                     list: JSON.stringify(keys),
                     default: 'default'
@@ -127,12 +134,6 @@ var SystemConfigurationView = View.extend({
             JSON: window.JSON
         }));
 
-        this.$('input[title]').tooltip({
-            container: this.$el,
-            animation: false,
-            delay: { show: 200 }
-        });
-
         var enableCollectionCrreationPolicy = this.settings['core.collection_create_policy'] ? this.settings['core.collection_create_policy'].open : false;
 
         this.$('.g-plugin-switch')
@@ -178,7 +179,7 @@ var SystemConfigurationView = View.extend({
         if (this.$('.g-plugin-switch').bootstrapSwitch('state')) {
             settingValue = { open: this.$('.g-plugin-switch').bootstrapSwitch('state') };
             var accessList = this.accessWidget.getAccessList();
-            _.each(_.keys(accessList), key => {
+            _.each(_.keys(accessList), (key) => {
                 settingValue[key] = _.pluck(accessList[key], 'id');
             });
         } else {
@@ -190,4 +191,3 @@ var SystemConfigurationView = View.extend({
 });
 
 export default SystemConfigurationView;
-
