@@ -1,5 +1,5 @@
 import View from 'girder/views/View';
-import { apiRoot } from 'girder/rest';
+import { getApiRoot } from 'girder/rest';
 
 import LayoutFooterTemplate from 'girder/templates/layout/layoutFooter.pug';
 
@@ -9,13 +9,19 @@ import 'girder/stylesheets/layout/footer.styl';
  * This view shows the footer in the layout.
  */
 var LayoutFooterView = View.extend({
+
+    initialize: function (settings) {
+        const contactEmail = settings.contactEmail || null;
+        this.contactHref = contactEmail !== null ? `mailto:${contactEmail}` : null;
+    },
+
     render: function () {
         this.$el.html(LayoutFooterTemplate({
-            apiRoot: apiRoot
+            apiRoot: getApiRoot(),
+            contactHref: this.contactHref
         }));
         return this;
     }
 });
 
 export default LayoutFooterView;
-
