@@ -10,8 +10,6 @@ import events from 'girder/events';
 
 import GroupModListTemplate from 'girder/templates/widgets/groupModList.pug';
 
-import 'bootstrap/js/tooltip';
-
 /**
  * This view shows a list of moderators of a group.
  */
@@ -27,20 +25,18 @@ var GroupModsWidget = View.extend({
 
         'click .g-group-mod-demote': function (e) {
             var li = $(e.currentTarget).parents('li');
-            var view = this;
 
             confirm({
                 text: 'Are you sure you want to remove moderator privileges ' +
                     'from <b>' + _.escape(li.attr('username')) + '</b>?',
                 escapedHtml: true,
-                confirmCallback: function () {
-                    view.trigger('g:demoteUser', li.attr('userid'));
+                confirmCallback: () => {
+                    this.trigger('g:demoteUser', li.attr('userid'));
                 }
             });
         },
 
         'click a.g-group-mod-remove': function (e) {
-            var view = this;
             var li = $(e.currentTarget).parents('li');
 
             confirm({
@@ -48,8 +44,8 @@ var GroupModsWidget = View.extend({
                     _.escape(li.attr('username')) +
                     '</b> from this group?',
                 escapedHtml: true,
-                confirmCallback: function () {
-                    view.trigger('g:removeMember', li.attr('userid'));
+                confirmCallback: () => {
+                    this.trigger('g:removeMember', li.attr('userid'));
                 }
             });
         },
@@ -74,13 +70,6 @@ var GroupModsWidget = View.extend({
             accessType: AccessType
         }));
 
-        this.$('.g-group-mod-demote.g-group-mod-promote,.g-group-mod-remove').tooltip({
-            container: 'body',
-            placement: 'left',
-            animation: false,
-            delay: {show: 100}
-        });
-
         return this;
     }
 });
