@@ -19,6 +19,19 @@ var ItemListWidget = View.extend({
         },
         'click a.g-show-more-items': function () {
             this.collection.fetchNextPage();
+        },
+        'change .g-list-checkbox': function (event) {
+            const target = $(event.currentTarget);
+            const cid = target.attr('g-item-cid');
+            if (target.prop('checked')) {
+                this.checked.push(cid);
+            } else {
+                const idx = this.checked.indexOf(cid);
+                if (idx !== -1) {
+                    this.checked.splice(idx, 1);
+                }
+            }
+            this.trigger('g:checkboxesChanged');
         }
     },
 
@@ -32,6 +45,7 @@ var ItemListWidget = View.extend({
         this._showSizes = (
           _.has(settings, 'showSizes') ? settings.showSizes : true);
         this.accessLevel = settings.accessLevel;
+        this.public = settings.public;
 
         new LoadingAnimation({
             el: this.$el,
@@ -57,6 +71,7 @@ var ItemListWidget = View.extend({
         this.checked = [];
         this.$el.html(ItemListTemplate({
             items: this.collection.toArray(),
+            isParentPublic: this.public,
             hasMore: this.collection.hasNextPage(),
             formatSize: formatSize,
             checkboxes: this._checkboxes,
@@ -65,25 +80,6 @@ var ItemListWidget = View.extend({
             showSizes: this._showSizes
         }));
 
-        this.$('.g-item-list-entry a[title]').tooltip({
-            container: 'body',
-            placement: 'auto',
-            delay: 100
-        });
-
-        var view = this;
-        this.$('.g-list-checkbox').change(function () {
-            var cid = $(this).attr('g-item-cid');
-            if (this.checked) {
-                view.checked.push(cid);
-            } else {
-                var idx = view.checked.indexOf(cid);
-                if (idx !== -1) {
-                    view.checked.splice(idx, 1);
-                }
-            }
-            view.trigger('g:checkboxesChanged');
-        });
         return this;
     },
 
