@@ -8,15 +8,11 @@ import PaginateWidgetTemplate from 'girder/templates/widgets/paginateWidget.pug'
  */
 var PaginateWidget = View.extend({
     events: {
-        'click .g-page-next': function (e) {
-            if (!$(e.currentTarget).hasClass('disabled')) {
-                this.collection.fetchNextPage();
-            }
+        'click .g-page-next:not(.disabled)': function (e) {
+            this.collection.fetchNextPage();
         },
-        'click .g-page-prev': function (e) {
-            if (!$(e.currentTarget).hasClass('disabled')) {
-                this.collection.fetchPreviousPage();
-            }
+        'click .g-page-prev:not(.disabled)': function (e) {
+            this.collection.fetchPreviousPage();
         }
     },
 
@@ -32,17 +28,8 @@ var PaginateWidget = View.extend({
             collection: this.collection
         }));
 
-        if (this.collection.hasNextPage()) {
-            this.$('.g-page-next').removeClass('disabled');
-        } else {
-            this.$('.g-page-next').addClass('disabled');
-        }
-
-        if (this.collection.hasPreviousPage()) {
-            this.$('.g-page-prev').removeClass('disabled');
-        } else {
-            this.$('.g-page-prev').addClass('disabled');
-        }
+        this.$('.g-page-next').girderEnable(this.collection.hasNextPage());
+        this.$('.g-page-prev').girderEnable(this.collection.hasPreviousPage());
         return this;
     }
 });
