@@ -6,8 +6,6 @@ import TimelineTemplate from 'girder/templates/widgets/timeline.pug';
 
 import 'girder/stylesheets/widgets/timelineWidget.styl';
 
-import 'bootstrap/js/tooltip';
-
 /**
  * This widget displays a timeline of events. This is visualized as a line (a bar)
  * with two sorts of primitives overlaid:
@@ -93,10 +91,7 @@ var TimelineWidget = View.extend({
             var start = this.numeric ? segment.start : new Date(segment.start);
             var end = this.numeric ? segment.end : new Date(segment.end);
             var classes = segment.class ? [segment.class] : [this.defaultSegmentClass];
-
-            if (segment.tooltip) {
-                classes.push('g-tooltip');
-            }
+            var color = segment.color ? `background-color: ${segment.color}` : '';
 
             return {
                 class: classes.join(' '),
@@ -104,24 +99,23 @@ var TimelineWidget = View.extend({
                 width: (100 * (end - start) / range).toFixed(1) + '%',
                 tooltip: this._segmentTooltip(segment, {
                     range: this.numeric ? end - start : (end - start) / 1000
-                })
+                }),
+                color
             };
         }, this);
 
         this._processedPoints = _.map(this.points, function (point) {
             var time = this.numeric ? point.time : new Date(point.time);
             var classes = point.class ? [point.class] : [this.defaultPointClass];
-
-            if (point.tooltip) {
-                classes.push('g-tooltip');
-            }
+            var color = point.color ? `background-color: ${point.color}` : '';
 
             return {
                 class: classes.join(' '),
                 left: (100 * (time - this.startTime) / range).toFixed(1) + '%',
                 tooltip: this._pointTooltip(point, {
                     time: time
-                })
+                }),
+                color
             };
         }, this);
     },
@@ -154,14 +148,8 @@ var TimelineWidget = View.extend({
             endLabel: this.endLabel
         }));
 
-        this.$('.g-tooltip').tooltip({
-            delay: 100,
-            container: this.$el
-        });
-
         return this;
     }
 });
 
 export default TimelineWidget;
-
