@@ -25,7 +25,7 @@ from .model_base import Model, ValidationException
 from girder import events
 from girder.constants import AccessType, CoreEventHandler
 from girder.models.model_base import AccessControlledModel
-from girder.utility import assetstore_utilities, acl_mixin
+from girder.utility import acl_mixin
 
 
 class File(acl_mixin.AccessControlMixin, Model):
@@ -33,6 +33,8 @@ class File(acl_mixin.AccessControlMixin, Model):
     This model represents a File, which is stored in an assetstore.
     """
     def initialize(self):
+        from girder.utility import assetstore_utilities
+
         self.name = 'file'
         self.ensureIndices(
             ['itemId', 'assetstoreId', 'exts'] +
@@ -61,11 +63,13 @@ class File(acl_mixin.AccessControlMixin, Model):
             to False if you plan to delete the item and do not care about
             updating its size.
         """
+        from .item import Item
+
         if file.get('assetstoreId'):
             self.getAssetstoreAdapter(file).deleteFile(file)
 
         if file['itemId']:
-            item = self.model('item').load(file['itemId'], force=True)
+            item = Item().load(file['itemId'], force=True)
             # files that are linkUrls might not have a size field
             if 'size' in file:
                 self.propagateSizeChange(item, -file['size'], updateItemSize)
@@ -93,19 +97,54 @@ class File(acl_mixin.AccessControlMixin, Model):
         :type contentDisposition: str or None
         :type extraParameters: str or None
         """
+        events.trigger('model.file.download.request', info={
+            'file': file,
+            'startByte': offset,
+            'endByte': endByte})
+
         if file.get('assetstoreId'):
-            return self.getAssetstoreAdapter(file).downloadFile(
-                file, offset=offset, headers=headers, endByte=endByte,
-                contentDisposition=contentDisposition,
-                extraParameters=extraParameters)
+            try:
+                fileDownload = self.getAssetstoreAdapter(file).downloadFile(
+                    file, offset=offset, headers=headers, endByte=endByte,
+                    contentDisposition=contentDisposition,
+                    extraParameters=extraParameters)
+
+                def downloadGenerator():
+                    for data in fileDownload():
+                        yield data
+                    if endByte is None or endByte >= file['size']:
+                        events.trigger('model.file.download.complete', info={
+                            'file': file,
+                            'startByte': offset,
+                            'endByte': endByte,
+                            'redirect': False})
+                return downloadGenerator
+            except cherrypy.HTTPRedirect:
+                events.trigger('model.file.download.complete', info={
+                    'file': file,
+                    'startByte': offset,
+                    'endByte': endByte,
+                    'redirect': True})
+                raise
         elif file.get('linkUrl'):
             if headers:
+                events.trigger('model.file.download.complete', info={
+                    'file': file,
+                    'startByte': offset,
+                    'endByte': endByte,
+                    'redirect': True})
                 raise cherrypy.HTTPRedirect(file['linkUrl'])
             else:
                 endByte = endByte or len(file['linkUrl'])
 
                 def stream():
                     yield file['linkUrl'][offset:endByte]
+                    if endByte >= len(file['linkUrl']):
+                        events.trigger('model.file.download.complete', info={
+                            'file': file,
+                            'startByte': offset,
+                            'endByte': endByte,
+                            'redirect': False})
                 return stream
         else:  # pragma: no cover
             raise Exception('File has no known download mechanism.')
@@ -129,7 +168,8 @@ class File(acl_mixin.AccessControlMixin, Model):
 
         return doc
 
-    def createLinkFile(self, name, parent, parentType, url, creator, size=None, mimeType=None):
+    def createLinkFile(self, name, parent, parentType, url, creator, size=None,
+                       mimeType=None, reuseExisting=False):
         """
         Create a file that is a link to a URL, rather than something we maintain
         in an assetstore.
@@ -147,33 +187,53 @@ class File(acl_mixin.AccessControlMixin, Model):
         :type size: int
         :param mimeType: The mimeType of the file. (optional)
         :type mimeType: str
+        :param reuseExisting: If a file with the same name already exists in
+            this location, return it rather than creating a new file.
+        :type reuseExisting: bool
         """
+        from .item import Item
+
         if parentType == 'folder':
             # Create a new item with the name of the file.
-            item = self.model('item').createItem(
-                name=name, creator=creator, folder=parent)
+            item = Item().createItem(
+                name=name, creator=creator, folder=parent, reuseExisting=reuseExisting)
         elif parentType == 'item':
             item = parent
 
-        file = {
-            'created': datetime.datetime.utcnow(),
-            'itemId': item['_id'],
+        existing = None
+        if reuseExisting:
+            existing = self.findOne({
+                'itemId': item['_id'],
+                'name': name
+            })
+
+        if existing:
+            file = existing
+        else:
+            file = {
+                'created': datetime.datetime.utcnow(),
+                'itemId': item['_id'],
+                'assetstoreId': None,
+                'name': name
+            }
+
+        file.update({
             'creatorId': creator['_id'],
-            'assetstoreId': None,
-            'name': name,
             'mimeType': mimeType,
             'linkUrl': url
-        }
-
+        })
         if size is not None:
             file['size'] = int(size)
 
         try:
-            file = self.save(file)
+            if existing:
+                file = self.updateFile(file)
+            else:
+                file = self.save(file)
             return file
         except ValidationException:
             if parentType == 'folder':
-                self.model('item').remove(item)
+                Item().remove(item)
             raise
 
     def propagateSizeChange(self, item, sizeIncrement, updateItemSize=True):
@@ -192,14 +252,17 @@ class File(acl_mixin.AccessControlMixin, Model):
             False if you plan to delete the item immediately and don't care to
             update its size.
         """
+        from .folder import Folder
+        from .item import Item
+
         if updateItemSize:
             # Propagate size up to item
-            self.model('item').increment(query={
+            Item().increment(query={
                 '_id': item['_id']
             }, field='size', amount=sizeIncrement, multi=False)
 
         # Propagate size to direct parent folder
-        self.model('folder').increment(query={
+        Folder().increment(query={
             '_id': item['folderId']
         }, field='size', amount=sizeIncrement, multi=False)
 
@@ -264,10 +327,12 @@ class File(acl_mixin.AccessControlMixin, Model):
         # certain that the file will actually be saved. It is also possible for
         # "model.file.save" to set "defaultPrevented", which would prevent the
         # item from being saved initially.
+        from .item import Item
+
         fileDoc = event.info
         itemId = fileDoc.get('itemId')
         if itemId and fileDoc.get('size'):
-            item = self.model('item').load(itemId, force=True)
+            item = Item().load(itemId, force=True)
             self.propagateSizeChange(item, fileDoc['size'])
 
     def updateFile(self, file):
@@ -288,7 +353,10 @@ class File(acl_mixin.AccessControlMixin, Model):
         """
         Return the assetstore adapter for the given file.
         """
-        assetstore = self.model('assetstore').load(file['assetstoreId'])
+        from .assetstore import Assetstore
+        from girder.utility import assetstore_utilities
+
+        assetstore = Assetstore().load(file['assetstoreId'])
         return assetstore_utilities.getAssetstoreAdapter(assetstore)
 
     def copyFile(self, srcFile, creator, item=None):
@@ -343,8 +411,8 @@ class File(acl_mixin.AccessControlMixin, Model):
                 attachedDoc = modelType.load(
                     file.get('attachedToId'))
         else:
-            attachedDoc = self.model('item').load(
-                file.get('itemId'), force=True)
+            from .item import Item
+            attachedDoc = Item().load(file.get('itemId'), force=True)
         return not attachedDoc
 
     def updateSize(self, file):
@@ -365,7 +433,7 @@ class File(acl_mixin.AccessControlMixin, Model):
         system file with ``'rb'`` mode. This can also be used as a context
         manager, e.g.:
 
-        >>> with self.model('file').open(file) as fh:
+        >>> with File().open(file) as fh:
         >>>    while True:
         >>>        chunk = fh.read(CHUNK_LEN)
         >>>        if not chunk:
