@@ -27,7 +27,7 @@ import six
 from bson.objectid import ObjectId
 from bson.errors import InvalidId
 from pymongo.errors import WriteError
-from girder import events, logprint
+from girder import events, logprint, logger
 from girder.constants import AccessType, CoreEventHandler, ACCESS_FLAGS, TEXT_SCORE_SORT_MAX
 from girder.external.mongodb_proxy import MongoProxy
 from girder.models import getDbConnection
@@ -36,8 +36,25 @@ from girder.utility.model_importer import ModelImporter
 # pymongo3 complains about extra kwargs to find(), so we must filter them.
 _allowedFindArgs = ('cursor_type', 'allow_partial_results', 'oplog_replay',
                     'modifiers', 'manipulate')
+# This list is only used for testing, where we must reconnect() all models after
+# the database is dropped between each test case. If we find a cleverer way to do
+# that, we don't need to store these here.
+_modelSingletons = []
 
 
+class _ModelSingleton(type):
+    def __init__(cls, name, bases, dict):
+        super(_ModelSingleton, cls).__init__(name, bases, dict)
+        cls._instance = None
+
+    def __call__(cls, *args, **kwargs):
+        if cls._instance is None:
+            cls._instance = super(_ModelSingleton, cls).__call__(*args, **kwargs)
+            _modelSingletons.append(cls._instance)
+        return cls._instance
+
+
+@six.add_metaclass(_ModelSingleton)
 class Model(ModelImporter):
     """
     Model base class. Models are responsible for abstracting away the
@@ -569,15 +586,15 @@ class Model(ModelImporter):
         """
         return 1
 
-    def _isInclusionProjection(self, fields):
+    @staticmethod
+    def _isInclusionProjection(fields):
         """
         Test whether a projection filter is an inclusion filter (whitelist) or exclusion
         projection (blacklist) of fields, as defined by MongoDB find() method `projection` param.
 
         :param fields: A mask for filtering result documents by key, or None to return the full
             document, passed to MongoDB find() as the `projection` param.
-        :type fields: `str, list of strings or tuple of strings for fields to be included from the
-            document, or dict for an inclusion or exclusion projection`.
+        :type fields: list or dict or None
         """
         if fields is None:
             return False
@@ -596,6 +613,68 @@ class Model(ModelImporter):
         # Empty dict or just _id field
         return fields.get('_id', True)
 
+    @staticmethod
+    def _supplementFields(fields, overwrite):
+        """
+        Overwrite the projection filter to either include (in the case of an inclusion filter) or
+        not exclude (in the case of an exclusion filter) the contents of overwrite.
+
+        :param fields: A mask for filtering result documents by key, or None to return the full
+            document, passed to MongoDB find() as the `projection` param.
+        :type fields: list or dict or None
+        :param overwrite: Additional document key(s) to be included or not excluded in fields.
+        :type overwrite: set
+        :returns: A copy of fields with the relevant overwrite changes.
+        """
+        if fields is None:
+            return fields
+
+        if Model._isInclusionProjection(fields):
+            if not isinstance(fields, dict):
+                # Inclusion projection (str, list, or tuple)
+                copy = list(set(fields) | overwrite)
+            else:
+                # Inclusion projection (dict)
+                copy = dict(fields)
+                copy.update(dict.fromkeys(overwrite, True))
+        else:
+            # Exclusion projection (dict)
+            copy = dict(fields)
+            for entry in overwrite:
+                copy.pop(entry, None)
+        return copy
+
+    @staticmethod
+    def _removeSupplementalFields(doc, fields):
+        """
+        Edit the document to be consistent with what the user originally requested, undoing what may
+        have been overwritten by _supplementFields().
+
+        :param doc: A document returned by MongoDB find()
+        :type doc: dict
+        :param fields: The original mask for filtering result documents by key, as specified by the
+            user to be passed to MongoDB find() as the `projection` param.
+        :type fields: list or dict or None
+        """
+        if fields is None:
+            return
+
+        whitelist = []
+        if isinstance(fields, dict):
+            for k, v in six.viewitems(fields):
+                if not v:
+                    doc.pop(k, None)
+                else:
+                    whitelist.append(k)
+            if whitelist:
+                for k in list(six.viewkeys(doc)):
+                    if k not in whitelist and k != '_id':
+                        del doc[k]
+        else:
+            for k in list(six.viewkeys(doc)):
+                if k not in fields and k != '_id':
+                    del doc[k]
+
 
 class AccessControlledModel(Model):
     """
@@ -760,10 +839,8 @@ class AccessControlledModel(Model):
         if entity not in doc['access']:
             doc['access'][entity] = []
 
-        # First remove any existing permission level for this entity.
-        doc['access'][entity] = [perm for perm in doc['access'][entity]
-                                 if perm['id'] != id]
-
+        key = 'access.' + entity
+        update = {}
         # Add in the new level for this entity unless we are removing access.
         if level is not None:
             entry = {
@@ -772,13 +849,50 @@ class AccessControlledModel(Model):
                 'flags': flags
             }
             entry['flags'] = self._validateFlags(doc, user, entity, entry, force)
-            doc['access'][entity].append(entry)
+            # because we're iterating this operation is not necessarily atomic
+            for index, perm in enumerate(doc['access'][entity]):
+                if perm['id'] == id:
+                    # if the id already exists we want to update with a $set
+                    doc['access'][entity][index] = entry
+                    update['$set'] = {'%s.%s' % (key, index): entry}
+                    break
+            else:
+                doc['access'][entity].append(entry)
+                update['$push'] = {key: entry}
+        # set remove query
+        else:
+            update['$pull'] = {key: {'id': id}}
+            for perm in doc['access'][entity]:
+                if perm['id'] == id:
+                    doc['access'][entity].remove(perm)
 
         if save:
-            doc = self.save(doc)
+            doc = self._saveAcl(doc, update)
 
         return doc
 
+    def _saveAcl(self, doc, update):
+        if '_id' not in doc:
+            return self.save(doc)
+
+        # copy all other (potentially updated) fields to the update list,
+        # and trigger normal save events
+        if '$set' in update:
+            for propKey in doc:
+                if propKey != 'access':
+                    update['$set'][propKey] = doc[propKey]
+        else:
+            update['$set'] = {k: v for k, v in six.viewitems(doc)
+                              if k != 'access'}
+
+        event = events.trigger('model.%s.save' % self.name, doc)
+        if not event.defaultPrevented:
+            doc = self.collection.find_one_and_update(
+                {'_id': ObjectId(doc['_id'])}, update,
+                return_document=pymongo.ReturnDocument.AFTER)
+            events.trigger('model.%s.save.after' % self.name, doc)
+        return doc
+
     def setPublic(self, doc, public, save=False):
         """
         Set the flag for public read access on the object.
@@ -1028,6 +1142,9 @@ class AccessControlledModel(Model):
         :type doc: dict
         :returns: A dict containing `users` and `groups` keys.
         """
+        from .user import User
+        from .group import Group
+
         acList = {
             'users': doc.get('access', {}).get('users', []),
             'groups': doc.get('access', {}).get('groups', [])
@@ -1036,9 +1153,7 @@ class AccessControlledModel(Model):
         dirty = False
 
         for user in acList['users'][:]:
-            userDoc = self.model('user').load(
-                user['id'], force=True,
-                fields=['firstName', 'lastName', 'login'])
+            userDoc = User().load(user['id'], force=True, fields=['firstName', 'lastName', 'login'])
             if not userDoc:
                 dirty = True
                 acList['users'].remove(user)
@@ -1047,8 +1162,7 @@ class AccessControlledModel(Model):
             user['name'] = ' '.join((userDoc['firstName'], userDoc['lastName']))
 
         for grp in acList['groups'][:]:
-            grpDoc = self.model('group').load(
-                grp['id'], force=True, fields=['name', 'description'])
+            grpDoc = Group().load(grp['id'], force=True, fields=['name', 'description'])
             if not grpDoc:
                 dirty = True
                 acList['groups'].remove(grp)
@@ -1216,40 +1330,35 @@ class AccessControlledModel(Model):
         :type user: dict or None
         :param objectId: Whether the id should be coerced to ObjectId type.
         :type objectId: bool
-        :param force: If you explicitly want to circumvent access
-                      checking on this resource, set this to True.
+        :param force: If you explicitly want to circumvent access checking on this resource, set
+            this to True.
         :type force: bool
         :param fields: A mask for filtering result documents by key, or None to return the full
             document, passed to MongoDB find() as the `projection` param.
-        :type fields: `str, list of strings or tuple of strings for fields to be included from the
-            document, or dict for an inclusion or exclusion projection`.
-        :param exc: If not found, throw a ValidationException instead of
-            returning None.
+        :type fields: list or dict
+        :param exc: If not found, throw a ValidationException instead of returning None.
         :type exc: bool
         :raises ValidationException: If an invalid ObjectId is passed.
         :returns: The matching document, or None if no match exists.
         """
+        # Warn of str type deprecation for `fields` param
+        if isinstance(fields, six.string_types):
+            logger.warning('String data type for fields param is deprecated, \
+                use a list or dict instead.')
+            fields = [fields]
 
         # Ensure we include access and public, they are needed by requireAccess
-        loadFields = copy.copy(fields)
-        if not force and self._isInclusionProjection(fields):
-            if isinstance(loadFields, dict):
-                loadFields['access'] = True
-                loadFields['public'] = True
-            else:
-                loadFields = list(set(loadFields) | {'access', 'public'})
+        loadFields = fields
+        if not force:
+            extraFields = {'access', 'public'}
+            loadFields = self._supplementFields(fields, extraFields)
 
         doc = Model.load(self, id=id, objectId=objectId, fields=loadFields, exc=exc)
 
         if not force and doc is not None:
             self.requireAccess(doc, user, level)
 
-            if fields is not None:
-                if 'access' not in fields:
-                    del doc['access']
-
-                if 'public' not in fields:
-                    del doc['public']
+            self._removeSupplementalFields(doc, fields)
 
         return doc
 
