@@ -21,12 +21,12 @@ from collections import OrderedDict
 import cherrypy
 import pymongo
 import six
+import re
 
 from ..constants import GIRDER_ROUTE_ID, GIRDER_STATIC_ROUTE_ID, SettingDefault, SettingKey
 from .model_base import Model, ValidationException
 from girder import logprint
-from girder.utility import plugin_utilities, setting_utilities
-from girder.utility.model_importer import ModelImporter
+from girder.utility import config, setting_utilities
 from bson.objectid import ObjectId
 
 
@@ -164,6 +164,31 @@ class Setting(Model):
                 return fn()
         return None
 
+    @staticmethod
+    @setting_utilities.validator(SettingKey.BRAND_NAME)
+    def validateCoreBrandName(doc):
+        if not doc['value']:
+            raise ValidationException('The brand name may not be empty', 'value')
+
+    @staticmethod
+    @setting_utilities.validator(SettingKey.BANNER_COLOR)
+    def validateCoreBannerColor(doc):
+        if not doc['value']:
+            raise ValidationException('The banner color may not be empty', 'value')
+        elif not (re.match(r'^#[0-9A-Fa-f]{6}$', doc['value'])):
+            raise ValidationException('The banner color must be a hex color triplet', 'value')
+
+    @staticmethod
+    @setting_utilities.validator(SettingKey.SECURE_COOKIE)
+    def validateSecureCookie(doc):
+        if not isinstance(doc['value'], bool):
+            raise ValidationException('Secure cookie option must be boolean.', 'value')
+
+    @staticmethod
+    @setting_utilities.default(SettingKey.SECURE_COOKIE)
+    def defaultSecureCookie():
+        return config.getConfig()['server']['mode'] == 'production'
+
     @staticmethod
     @setting_utilities.validator(SettingKey.PLUGINS_ENABLED)
     def validateCorePluginsEnabled(doc):
@@ -172,6 +197,8 @@ class Setting(Model):
         names. Removes any invalid plugin names, removes duplicates, and adds
         all transitive dependencies to the enabled list.
         """
+        from girder.utility import plugin_utilities
+
         if not isinstance(doc['value'], list):
             raise ValidationException('Plugins enabled setting must be a list.', 'value')
 
@@ -190,17 +217,20 @@ class Setting(Model):
     @staticmethod
     @setting_utilities.validator(SettingKey.COLLECTION_CREATE_POLICY)
     def validateCoreCollectionCreatePolicy(doc):
+        from .group import Group
+        from .user import User
+
         value = doc['value']
 
         if not isinstance(value, dict):
             raise ValidationException('Collection creation policy must be a JSON object.')
 
         for i, groupId in enumerate(value.get('groups', ())):
-            ModelImporter.model('group').load(groupId, force=True, exc=True)
+            Group().load(groupId, force=True, exc=True)
             value['groups'][i] = ObjectId(value['groups'][i])
 
         for i, userId in enumerate(value.get('users', ())):
-            ModelImporter.model('user').load(userId, force=True, exc=True)
+            User().load(userId, force=True, exc=True)
             value['users'][i] = ObjectId(value['users'][i])
 
         value['open'] = value.get('open', False)
@@ -267,6 +297,12 @@ class Setting(Model):
         if not doc['value']:
             raise ValidationException('Email from address must not be blank.', 'value')
 
+    @staticmethod
+    @setting_utilities.validator(SettingKey.CONTACT_EMAIL_ADDRESS)
+    def validateCoreContactEmailAddress(doc):
+        if not doc['value']:
+            raise ValidationException('Contact email address must not be blank.', 'value')
+
     @staticmethod
     @setting_utilities.validator(SettingKey.EMAIL_HOST)
     def validateCoreEmailHost(doc):
@@ -306,7 +342,7 @@ class Setting(Model):
         nonEmptyRoutes = [route for route in doc['value'].values() if route]
         for key in [GIRDER_ROUTE_ID, GIRDER_STATIC_ROUTE_ID]:
             if key not in doc['value'] or not doc['value'][key]:
-                raise ValidationException('Girder and static root must be routeable.')
+                raise ValidationException('Girder and static root must be routable.')
 
         for key in doc['value']:
             if (key != GIRDER_STATIC_ROUTE_ID and doc['value'][key] and
