@@ -1,5 +1,6 @@
 import View from 'girder/views/View';
 import { restRequest } from 'girder/rest';
+
 import template from '../templates/authorizeUpload.pug';
 import '../stylesheets/authorizeUpload.styl';
 
@@ -7,13 +8,13 @@ var AuthorizeUploadView = View.extend({
     events: {
         'click .g-create-authorized-upload': function () {
             restRequest({
-                path: 'authorized_upload',
-                type: 'POST',
+                url: 'authorized_upload',
+                method: 'POST',
                 data: {
                     folderId: this.folder.id,
                     duration: this.$('.g-num-days').val() || 30
                 }
-            }).done(data => {
+            }).done((data) => {
                 this.$('.g-url-container').removeClass('hide');
                 this.$('.g-authorized-upload-url-target').val(data.url).select();
             });
@@ -29,6 +30,7 @@ var AuthorizeUploadView = View.extend({
         this.$el.html(template({
             folder: this.folder
         }));
+        return this;
     }
 });
 
