@@ -8,13 +8,13 @@ import { restRequest } from 'girder/rest';
  */
 AssetstoreModel.hdfsImport = function (params) {
     restRequest({
-        path: 'hdfs_assetstore/' + this.get('_id') + '/import',
-        type: 'PUT',
+        url: `hdfs_assetstore/${this.id}/import`,
+        method: 'PUT',
         data: params,
         error: null
     }).done(_.bind(function () {
         this.trigger('g:imported');
-    }, this)).error(_.bind(function (err) {
+    }, this)).fail(_.bind(function (err) {
         this.trigger('g:error', err);
     }, this));
 
