@@ -9,8 +9,8 @@ import { wrap } from 'girder/utilities/PluginUtils';
  */
 wrap(HierarchyWidget, 'uploadDialog', function (uploadDialog) {
     restRequest({
-        type: 'GET',
-        path: 'item/licenses'
+        method: 'GET',
+        url: 'item/licenses'
     }).done(_.bind(function (resp) {
         this.licenses = resp;
         uploadDialog.call(this);
@@ -24,8 +24,8 @@ wrap(HierarchyWidget, 'uploadDialog', function (uploadDialog) {
  */
 wrap(HierarchyWidget, 'createItemDialog', function (createItemDialog) {
     restRequest({
-        type: 'GET',
-        path: 'item/licenses'
+        method: 'GET',
+        url: 'item/licenses'
     }).done(_.bind(function (resp) {
         this.licenses = resp;
         createItemDialog.call(this);
