@@ -1,3 +1,5 @@
+import $ from 'jquery';
+
 import View from 'girder/views/View';
 
 import panel from '../templates/panel.pug';
@@ -14,6 +16,11 @@ var Panel = View.extend({
     },
     render: function () {
         this.$el.html(panel(this.spec));
+
+        // initialize collapsible elements
+        this.$('.g-panel-content').collapse({toggle: false});
+
+        return this;
     },
     expand: function () {
         this.$('.icon-down-open').attr('class', 'icon-up-open');
