@@ -1,3 +1,5 @@
+/* eslint-disable import/first */
+
 import router from 'girder/router';
 import events from 'girder/events';
 
@@ -6,25 +8,33 @@ import JobDetailsWidget from './views/JobDetailsWidget';
 import JobListWidget from './views/JobListWidget';
 
 router.route('job/:id', 'jobView', function (id) {
-    var job = new JobModel({_id: id}).once('g:fetched', function () {
+    var job = new JobModel({ _id: id }).once('g:fetched', function () {
         events.trigger('g:navigateTo', JobDetailsWidget, {
             job: job,
             renderImmediate: true
         });
     }, this).once('g:error', function () {
-        router.navigate('collections', {trigger: true});
+        router.navigate('collections', { trigger: true });
     }, this);
     job.fetch();
 });
 
-router.route('jobs/user/:id', 'jobList', function (id) {
+router.route('jobs/user/:id(/:view)', 'jobList', function (id, view) {
     events.trigger('g:navigateTo', JobListWidget, {
-        filter: {userId: id}
+        filter: { userId: id },
+        view: view,
+        showGraphs: true,
+        showFilters: true,
+        showPageSizeSelector: true
     });
 });
 
-router.route('jobs', 'allJobList', function () {
+router.route('jobs(/:view)', 'allJobList', function (view) {
     events.trigger('g:navigateTo', JobListWidget, {
-        allJobsMode: true
+        allJobsMode: true,
+        view: view,
+        showGraphs: true,
+        showFilters: true,
+        showPageSizeSelector: true
     });
 });
