@@ -19,7 +19,8 @@
 
 from girder import events
 from girder.constants import SettingDefault, SortDir
-from girder.models.model_base import ModelImporter, ValidationException
+from girder.models.model_base import ValidationException
+from girder.models.user import User
 from girder.utility import setting_utilities
 from . import rest, constants, providers
 
@@ -30,6 +31,12 @@ def validateProvidersEnabled(doc):
         raise ValidationException('The enabled providers must be a list.', 'value')
 
 
+@setting_utilities.validator(constants.PluginSettings.IGNORE_REGISTRATION_POLICY)
+def validateIgnoreRegistrationPolicy(doc):
+    if not isinstance(doc['value'], bool):
+        raise ValidationException('Ignore registration policy setting must be boolean.', 'value')
+
+
 @setting_utilities.validator({
     constants.PluginSettings.GOOGLE_CLIENT_ID,
     constants.PluginSettings.GLOBUS_CLIENT_ID,
@@ -69,10 +76,10 @@ def checkOauthUser(event):
 
 
 def load(info):
-    ModelImporter.model('user').ensureIndex((
+    User().ensureIndex((
         (('oauth.provider', SortDir.ASCENDING),
          ('oauth.id', SortDir.ASCENDING)), {}))
-    ModelImporter.model('user').reconnect()
+    User().reconnect()
 
     events.bind('no_password_login_attempt', 'oauth', checkOauthUser)
 
