@@ -20,29 +20,27 @@
 from six.moves import urllib
 
 from girder.api.rest import getApiUrl, RestException
+from girder.models.setting import Setting
 from .base import ProviderBase
 from .. import constants
 
 
 class GitHub(ProviderBase):
     _AUTH_URL = 'https://github.com/login/oauth/authorize'
-    _AUTH_SCOPES = ('user:email',)
+    _AUTH_SCOPES = ['user:email']
     _TOKEN_URL = 'https://github.com/login/oauth/access_token'
     _API_USER_URL = 'https://api.github.com/user'
     _API_EMAILS_URL = 'https://api.github.com/user/emails'
 
     def getClientIdSetting(self):
-        return self.model('setting').get(
-            constants.PluginSettings.GITHUB_CLIENT_ID)
+        return Setting().get(constants.PluginSettings.GITHUB_CLIENT_ID)
 
     def getClientSecretSetting(self):
-        return self.model('setting').get(
-            constants.PluginSettings.GITHUB_CLIENT_SECRET)
+        return Setting().get(constants.PluginSettings.GITHUB_CLIENT_SECRET)
 
     @classmethod
     def getUrl(cls, state):
-        clientId = cls.model('setting').get(
-            constants.PluginSettings.GITHUB_CLIENT_ID)
+        clientId = Setting().get(constants.PluginSettings.GITHUB_CLIENT_ID)
 
         if clientId is None:
             raise Exception('No GitHub client ID setting is present.')
