@@ -20,29 +20,27 @@
 from six.moves import urllib
 
 from girder.api.rest import getApiUrl, RestException
+from girder.models.setting import Setting
 from .base import ProviderBase
 from .. import constants
 
 
 class Globus(ProviderBase):
     _AUTH_URL = 'https://auth.globus.org/v2/oauth2/authorize'
-    _AUTH_SCOPES = ('urn:globus:auth:scope:auth.globus.org:view_identities',
-                    'openid', 'profile', 'email')
+    _AUTH_SCOPES = ['urn:globus:auth:scope:auth.globus.org:view_identities',
+                    'openid', 'profile', 'email']
     _TOKEN_URL = 'https://auth.globus.org/v2/oauth2/token'
     _API_USER_URL = 'https://auth.globus.org/v2/oauth2/userinfo'
 
     def getClientIdSetting(self):
-        return self.model('setting').get(
-            constants.PluginSettings.GLOBUS_CLIENT_ID)
+        return Setting().get(constants.PluginSettings.GLOBUS_CLIENT_ID)
 
     def getClientSecretSetting(self):
-        return self.model('setting').get(
-            constants.PluginSettings.GLOBUS_CLIENT_SECRET)
+        return Setting().get(constants.PluginSettings.GLOBUS_CLIENT_SECRET)
 
     @classmethod
     def getUrl(cls, state):
-        clientId = cls.model('setting').get(
-            constants.PluginSettings.GLOBUS_CLIENT_ID)
+        clientId = Setting().get(constants.PluginSettings.GLOBUS_CLIENT_ID)
 
         if clientId is None:
             raise Exception('No Globus client ID setting is present.')
