@@ -21,10 +21,13 @@ import cherrypy
 import datetime
 import six
 
+from girder import events
 from girder.constants import AccessType
 from girder.api.describe import Description, autoDescribeRoute
 from girder.api.rest import Resource, RestException
 from girder.api import access
+from girder.models.setting import Setting
+from girder.models.token import Token
 from . import constants, providers
 
 
@@ -37,7 +40,7 @@ class OAuth(Resource):
         self.route('GET', (':provider', 'callback'), self.callback)
 
     def _createStateToken(self, redirect):
-        csrfToken = self.model('token').createToken(days=0.25)
+        csrfToken = Token().createToken(days=0.25)
 
         # The delimiter is arbitrary, but a dot doesn't need to be URL-encoded
         state = '%s.%s' % (csrfToken['_id'], redirect)
@@ -51,13 +54,11 @@ class OAuth(Resource):
         """
         csrfTokenId, _, redirect = state.partition('.')
 
-        token = self.model('token').load(
-            csrfTokenId, objectId=False, level=AccessType.READ)
+        token = Token().load(csrfTokenId, objectId=False, level=AccessType.READ)
         if token is None:
-            raise RestException('Invalid CSRF token (state="%s").' % state,
-                                code=403)
+            raise RestException('Invalid CSRF token (state="%s").' % state, code=403)
 
-        self.model('token').remove(token)
+        Token().remove(token)
 
         if token['expires'] < datetime.datetime.utcnow():
             raise RestException('Expired CSRF token (state="%s").' % state,
@@ -78,8 +79,8 @@ class OAuth(Resource):
         .param('list', 'Whether to return the providers as an ordered list.',
                required=False, dataType='boolean', default=False)
     )
-    def listProviders(self, redirect, list, params):
-        enabledNames = self.model('setting').get(constants.PluginSettings.PROVIDERS_ENABLED)
+    def listProviders(self, redirect, list):
+        enabledNames = Setting().get(constants.PluginSettings.PROVIDERS_ENABLED)
 
         enabledProviders = [
             provider
@@ -115,7 +116,7 @@ class OAuth(Resource):
         .param('error', 'Error message from provider.', required=False),
         hide=True
     )
-    def callback(self, provider, state, code, error, params):
+    def callback(self, provider, state, code, error):
         if error is not None:
             raise RestException("Provider returned error: '%s'." % error, code=502)
 
@@ -130,8 +131,24 @@ class OAuth(Resource):
 
         providerObj = provider(cherrypy.url())
         token = providerObj.getToken(code)
+
+        events.trigger('oauth.auth_callback.before', {
+            'provider': provider,
+            'token': token
+        })
+
         user = providerObj.getUser(token)
 
-        self.sendAuthTokenCookie(user)
+        events.trigger('oauth.auth_callback.after', {
+            'provider': provider,
+            'token': token,
+            'user': user
+        })
+
+        girderToken = self.sendAuthTokenCookie(user)
+        try:
+            redirect = redirect.format(girderToken=str(girderToken['_id']))
+        except KeyError:
+            pass  # in case there's another {} that's not handled by format
 
         raise cherrypy.HTTPRedirect(redirect)
