@@ -1,8 +1,9 @@
+import $ from 'jquery';
 import _ from 'underscore';
 
 import PluginConfigBreadcrumbWidget from 'girder/views/widgets/PluginConfigBreadcrumbWidget';
 import View from 'girder/views/View';
-import { apiRoot, restRequest } from 'girder/rest';
+import { getApiRoot, restRequest } from 'girder/rest';
 import events from 'girder/events';
 
 import ConfigViewTemplate from '../templates/configView.pug';
@@ -22,6 +23,24 @@ var ConfigView = View.extend({
                 key: 'oauth.' + providerId + '_client_secret',
                 value: this.$('#g-oauth-provider-' + providerId + '-client-secret').val().trim()
             }]);
+        },
+
+        'change .g-ignore-registration-policy': function (event) {
+            restRequest({
+                method: 'PUT',
+                url: 'system/setting',
+                data: {
+                    key: 'oauth.ignore_registration_policy',
+                    value: $(event.target).is(':checked')
+                }
+            }).done(() => {
+                events.trigger('g:alert', {
+                    icon: 'ok',
+                    text: 'Setting saved.',
+                    type: 'success',
+                    timeout: 3000
+                });
+            });
         }
     },
 
@@ -79,15 +98,15 @@ var ConfigView = View.extend({
         }];
         this.providerIds = _.pluck(this.providers, 'id');
 
-        var settingKeys = [];
+        var settingKeys = ['oauth.ignore_registration_policy'];
         _.each(this.providerIds, function (id) {
             settingKeys.push('oauth.' + id + '_client_id');
             settingKeys.push('oauth.' + id + '_client_secret');
         }, this);
 
         restRequest({
-            type: 'GET',
-            path: 'system/setting',
+            method: 'GET',
+            url: 'system/setting',
             data: {
                 list: JSON.stringify(settingKeys)
             }
@@ -99,10 +118,10 @@ var ConfigView = View.extend({
 
     render: function () {
         var origin = window.location.protocol + '//' + window.location.host,
-            _apiRoot = apiRoot;
+            _apiRoot = getApiRoot();
 
-        if (apiRoot.substring(0, 1) !== '/') {
-            _apiRoot = '/' + apiRoot;
+        if (_apiRoot.substring(0, 1) !== '/') {
+            _apiRoot = '/' + _apiRoot;
         }
 
         this.$el.html(ConfigViewTemplate({
@@ -126,6 +145,9 @@ var ConfigView = View.extend({
                 this.$('#g-oauth-provider-' + id + '-client-secret').val(
                     this.settingVals['oauth.' + id + '_client_secret']);
             }, this);
+
+            var checked = this.settingVals['oauth.ignore_registration_policy'];
+            this.$('.g-ignore-registration-policy').attr('checked', checked ? 'checked' : null);
         }
 
         return this;
@@ -140,8 +162,8 @@ var ConfigView = View.extend({
         });
 
         restRequest({
-            type: 'PUT',
-            path: 'system/setting',
+            method: 'PUT',
+            url: 'system/setting',
             data: {
                 list: JSON.stringify(settings)
             },
@@ -153,7 +175,7 @@ var ConfigView = View.extend({
                 type: 'success',
                 timeout: 3000
             });
-        }, this)).error(_.bind(function (resp) {
+        }, this)).fail(_.bind(function (resp) {
             this.$('#g-oauth-provider-' + providerId + '-error-message').text(
                 resp.responseJSON.message);
         }, this));
@@ -161,4 +183,3 @@ var ConfigView = View.extend({
 });
 
 export default ConfigView;
-
