@@ -1,3 +1,4 @@
+import $ from 'jquery';
 import _ from 'underscore';
 
 import View from 'girder/views/View';
@@ -22,7 +23,7 @@ var OAuthLoginView = View.extend({
         this.providers = null;
 
         restRequest({
-            path: 'oauth/provider',
+            url: 'oauth/provider',
             data: {
                 redirect: redirect,
                 list: true
@@ -35,7 +36,7 @@ var OAuthLoginView = View.extend({
 
     render: function () {
         if (this.providers === null) {
-            return;
+            return this;
         }
 
         var buttons = [];
@@ -57,6 +58,8 @@ var OAuthLoginView = View.extend({
                 buttons: buttons
             }));
         }
+
+        return this;
     },
 
     _buttons: {
