@@ -1,3 +1,5 @@
+import $ from 'jquery';
+
 import { AccessType } from 'girder/constants';
 import { getCurrentUser } from 'girder/auth';
 import { wrap } from 'girder/utilities/PluginUtils';
@@ -32,14 +34,16 @@ function extendView(View, Template, modelType) {
     });
 
     View.prototype.editPolicies = function () {
-        new QuotaPoliciesWidget({
+        var widget = new QuotaPoliciesWidget({
             el: $('#g-dialog-container'),
             model: this.model,
             modelType: modelType,
             parentView: this
         }).on('g:saved', function () {
             this.render();
-        }, this);
+        }, this).on('g:hidden', function () {
+            widget.destroy();
+        });
     };
 }
 
