@@ -1,11 +1,14 @@
+import $ from 'jquery';
+
+import vg from 'vega';
+
 import View from 'girder/views/View';
 import { AccessType } from 'girder/constants';
+import { restRequest } from 'girder/rest';
 
 import VegaWidgetTemplate from '../templates/vegaWidget.pug';
 import '../stylesheets/vegaWidget.styl';
 
-import vg from 'vega';
-
 var VegaWidget = View.extend({
     initialize: function (settings) {
         this.item = settings.item;
@@ -22,23 +25,23 @@ var VegaWidget = View.extend({
         if (this.accessLevel >= AccessType.READ && meta && meta.vega) {
             $('#g-app-body-container')
                 .append(VegaWidgetTemplate());
-            $.ajax({
-                url: '/api/v1/item/' + this.item.get('_id') + '/download',
-                type: 'GET',
-                dataType: 'json',
-                success: function (spec) {
+            restRequest({
+                url: `item/${this.item.id}/download`
+            })
+                .done(function (spec) {
                     vg.parse.spec(spec, function (chart) {
                         chart({
                             el: '.g-item-vega-vis',
                             renderer: 'svg'
                         }).update();
                     });
-                }
-            });
+                });
         } else {
             $('.g-item-vega')
                 .remove();
         }
+
+        return this;
     }
 });
 
