@@ -17,6 +17,7 @@
 #  limitations under the License.
 ###############################################################################
 
+import filelock
 from hashlib import sha512
 import os
 import psutil
@@ -29,6 +30,10 @@ import tempfile
 from girder import events, logger
 from girder.api.rest import setResponseHeader
 from girder.models.model_base import ValidationException, GirderException
+from girder.models.file import File
+from girder.models.folder import Folder
+from girder.models.item import Item
+from girder.models.upload import Upload
 from girder.utility import mkdir, progress
 from . import hash_state
 from .abstract_assetstore_adapter import AbstractAssetstoreAdapter
@@ -208,19 +213,29 @@ class FilesystemAssetstoreAdapter(AbstractAssetstoreAdapter):
         Moves the file into its permanent content-addressed location within the
         assetstore. Directory hierarchy yields 256^2 buckets.
         """
-        hash = hash_state.restoreHex(upload['sha512state'],
-                                     'sha512').hexdigest()
+        hash = hash_state.restoreHex(upload['sha512state'], 'sha512').hexdigest()
         dir = os.path.join(hash[0:2], hash[2:4])
         absdir = os.path.join(self.assetstore['root'], dir)
 
         path = os.path.join(dir, hash)
         abspath = os.path.join(self.assetstore['root'], path)
 
+        # Store the hash in the upload so that deleting a file won't delete
+        # this file
+        if '_id' in upload:
+            upload['sha512'] = hash
+            Upload().update({'_id': upload['_id']}, update={'$set': {'sha512': hash}})
+
         mkdir(absdir)
 
-        if os.path.exists(abspath):
+        # Only maintain the lock which checking if the file exists.  The only
+        # other place the lock is used is checking if an upload task has
+        # reserved the file, so this is sufficient.
+        with filelock.FileLock(abspath + '.deleteLock'):
+            pathExists = os.path.exists(abspath)
+        if pathExists:
             # Already have this file stored, just delete temp file.
-            os.remove(upload['tempFile'])
+            os.unlink(upload['tempFile'])
         else:
             # Move the temp file to permanent location in the assetstore.
             # shutil.move works across filesystems
@@ -292,18 +307,25 @@ class FilesystemAssetstoreAdapter(AbstractAssetstoreAdapter):
         Deletes the file from disk if it is the only File in this assetstore
         with the given sha512. Imported files are not actually deleted.
         """
-        if file.get('imported'):
+        from girder.models.file import File
+
+        if file.get('imported') or 'path' not in file:
             return
 
         q = {
             'sha512': file['sha512'],
             'assetstoreId': self.assetstore['_id']
         }
-        matching = self.model('file').find(q, limit=2, fields=[])
-        if matching.count(True) == 1:
-            path = os.path.join(self.assetstore['root'], file['path'])
-            if os.path.isfile(path):
-                os.remove(path)
+        path = os.path.join(self.assetstore['root'], file['path'])
+        if os.path.isfile(path):
+            with filelock.FileLock(path + '.deleteLock'):
+                matching = File().find(q, limit=2, fields=[])
+                matchingUpload = Upload().findOne(q)
+                if matching.count(True) == 1 and matchingUpload is None:
+                    try:
+                        os.unlink(path)
+                    except Exception:
+                        logger.exception('Failed to delete file %s' % path)
 
     def cancelUpload(self, upload):
         """
@@ -331,18 +353,17 @@ class FilesystemAssetstoreAdapter(AbstractAssetstoreAdapter):
         stat = os.stat(path)
         name = name or os.path.basename(path)
 
-        file = self.model('file').createFile(
-            name=name, creator=user, item=item, reuseExisting=True,
-            assetstore=self.assetstore, mimeType=mimeType, size=stat.st_size,
-            saveFile=False)
+        file = File().createFile(
+            name=name, creator=user, item=item, reuseExisting=True, assetstore=self.assetstore,
+            mimeType=mimeType, size=stat.st_size, saveFile=False)
         file['path'] = os.path.abspath(os.path.expanduser(path))
         file['mtime'] = stat.st_mtime
         file['imported'] = True
-        return self.model('file').save(file)
+        return File().save(file)
 
     def _importDataAsItem(self, name, user, folder, path, files, reuseExisting=True, params=None):
         params = params or {}
-        item = self.model('item').createItem(
+        item = Item().createItem(
             name=name, creator=user, folder=folder, reuseExisting=reuseExisting)
         events.trigger('filesystem_assetstore_imported',
                        {'id': item['_id'], 'type': 'item',
@@ -360,8 +381,7 @@ class FilesystemAssetstoreAdapter(AbstractAssetstoreAdapter):
             raise ValidationException(
                 'Files cannot be imported directly underneath a %s.' % parentType)
 
-        item = self.model('item').createItem(
-            name=name, creator=user, folder=parent, reuseExisting=True)
+        item = Item().createItem(name=name, creator=user, folder=parent, reuseExisting=True)
         events.trigger('filesystem_assetstore_imported', {
             'id': item['_id'],
             'type': 'item',
@@ -396,7 +416,7 @@ class FilesystemAssetstoreAdapter(AbstractAssetstoreAdapter):
                 if leafFoldersAsItems and self._hasOnlyFiles(path, localListDir):
                     self._importDataAsItem(name, user, parent, path, localListDir, params=params)
                 else:
-                    folder = self.model('folder').createFolder(
+                    folder = Folder().createFolder(
                         parent=parent, name=name, parentType=parentType,
                         creator=user, reuseExisting=True)
                     events.trigger(
@@ -437,7 +457,7 @@ class FilesystemAssetstoreAdapter(AbstractAssetstoreAdapter):
             'assetstoreId': self.assetstore['_id']
         }, **filters)
 
-        cursor = self.model('file').find(q)
+        cursor = File().find(q)
         progress.update(total=cursor.count(), current=0)
 
         for file in cursor:
