@@ -22,12 +22,15 @@ from hashlib import sha512
 import pymongo
 import six
 from six import BytesIO
+import time
 import uuid
 
 from girder import logger
 from girder.api.rest import setResponseHeader
+from girder.external.mongodb_proxy import MongoProxy
 from girder.models import getDbConnection
 from girder.models.model_base import ValidationException
+from girder.models.file import File
 from . import hash_state
 from .abstract_assetstore_adapter import AbstractAssetstoreAdapter
 
@@ -36,10 +39,17 @@ from .abstract_assetstore_adapter import AbstractAssetstoreAdapter
 # unless they are sending the final chunk.
 CHUNK_SIZE = 2097152
 
+# Cache recent connections so we can skip some start up actions
+RECENT_CONNECTION_CACHE_TIME = 600  # seconds
+RECENT_CONNECTION_CACHE_MAX_SIZE = 100
+_recentConnections = {}
+
 
 def _ensureChunkIndices(collection):
     """
     Ensure that we have appropriate indices on the chunk collection.
+
+    :param collection: a connection to a mongo collection.
     """
     collection.create_index([
         ('uuid', pymongo.ASCENDING),
@@ -47,6 +57,40 @@ def _ensureChunkIndices(collection):
     ], unique=True)
 
 
+def _setupSharding(collection, keyname='uuid'):
+    """
+    If we are communicating with a sharded server, and the collection is not
+    sharded, ask for it to be sharded based on a key.
+
+    :param collection: a connection to a mongo collection.
+    :param keyname: the name of the key to shard on.
+    :returns: True if sharding was added, False if it could not be added, or
+        'present' if already sharded.
+    """
+    database = collection.database
+    client = database.client
+    stat = client.admin.command('serverStatus')
+    # sharding will be non-None if the client is communicating with a mongos
+    # instance. For mongo 3.0 we have to check the process name for 'mongos'.
+    if not stat.get('sharding') and 'mongos' not in stat.get('process', ''):
+        return False
+    if database.command('collstats', collection.name).get('sharded'):
+        return 'present'
+    try:
+        client.admin.command('enableSharding', database.name)
+    except pymongo.errors.OperationFailure:
+        # sharding may already be enabled
+        pass
+    try:
+        client.admin.command(
+            'shardCollection', '%s.%s' % (database.name, collection.name),
+            key={keyname: 1})
+        return True
+    except pymongo.errors.OperationFailure:
+        pass
+    return False
+
+
 class GridFsAssetstoreAdapter(AbstractAssetstoreAdapter):
     """
     This assetstore type stores files within MongoDB using the GridFS data
@@ -67,8 +111,7 @@ class GridFsAssetstoreAdapter(AbstractAssetstoreAdapter):
 
         try:
             chunkColl = getDbConnection(
-                doc.get('mongohost', None), doc.get('replicaset', None),
-                autoRetry=False,
+                doc.get('mongohost'), doc.get('replicaset'), autoRetry=False,
                 serverSelectionTimeoutMS=10000)[doc['db']].chunk
             _ensureChunkIndices(chunkColl)
         except pymongo.errors.ServerSelectionTimeoutError as e:
@@ -79,31 +122,52 @@ class GridFsAssetstoreAdapter(AbstractAssetstoreAdapter):
 
     @staticmethod
     def fileIndexFields():
-        return ['sha512']
+        return ['sha512', 'chunkUuid']
 
     def __init__(self, assetstore):
         """
         :param assetstore: The assetstore to act on.
         """
         super(GridFsAssetstoreAdapter, self).__init__(assetstore)
+        recent = False
         try:
-            self.chunkColl = getDbConnection(
-                self.assetstore.get('mongohost', None),
-                self.assetstore.get('replicaset', None)
-            )[self.assetstore['db']].chunk
-            _ensureChunkIndices(self.chunkColl)
+            # Guard in case the connectionArgs is unhashable
+            key = (self.assetstore.get('mongohost'),
+                   self.assetstore.get('replicaset'),
+                   self.assetstore.get('shard'))
+            if key in _recentConnections:
+                recent = (time.time() - _recentConnections[key]['created'] <
+                          RECENT_CONNECTION_CACHE_TIME)
+        except TypeError:
+            key = None
+        try:
+            # MongoClient automatically reuses connections from a pool, but we
+            # want to avoid redoing ensureChunkIndices each time we get such a
+            # connection.
+            client = getDbConnection(self.assetstore.get('mongohost'),
+                                     self.assetstore.get('replicaset'),
+                                     quiet=recent)
+            self.chunkColl = MongoProxy(client[self.assetstore['db']].chunk)
+            if not recent:
+                _ensureChunkIndices(self.chunkColl)
+                if self.assetstore.get('shard') == 'auto':
+                    _setupSharding(self.chunkColl)
+                if key is not None:
+                    if len(_recentConnections) >= RECENT_CONNECTION_CACHE_MAX_SIZE:
+                        _recentConnections.clear()
+                    _recentConnections[key] = {
+                        'created': time.time()
+                    }
         except pymongo.errors.ConnectionFailure:
             logger.error('Failed to connect to GridFS assetstore %s',
                          self.assetstore['db'])
             self.chunkColl = 'Failed to connect'
             self.unavailable = True
-            return
         except pymongo.errors.ConfigurationError:
             logger.exception('Failed to configure GridFS assetstore %s',
                              self.assetstore['db'])
             self.chunkColl = 'Failed to configure'
             self.unavailable = True
-            return
 
     def initUpload(self, upload):
         """
@@ -130,31 +194,34 @@ class GridFsAssetstoreAdapter(AbstractAssetstoreAdapter):
         # Restore the internal state of the streaming SHA-512 checksum
         checksum = hash_state.restoreHex(upload['sha512state'], 'sha512')
 
-        # This bit of code will only do anything if there is a discrepancy
-        # between the received count of the upload record and the length of
-        # the file stored as chunks in the database. This code simply updates
-        # the sha512 state with the difference before reading the bytes sent
-        # from the user.
-        if self.requestOffset(upload) > upload['received']:
-            cursor = self.chunkColl.find({
-                'uuid': upload['chunkUuid'],
-                'n': {'$gte': upload['received'] // CHUNK_SIZE}
-            }, projection=['data']).sort('n', pymongo.ASCENDING)
-            for result in cursor:
-                checksum.update(result['data'])
-
-        cursor = self.chunkColl.find({
+        # TODO: when saving uploads is optional, we can conditionally try to
+        # fetch the last chunk.  Add these line before `lastChunk = ...`:
+        #   lastChunk = None
+        #   if '_id' in upload or upload['received'] != 0:
+        lastChunk = self.chunkColl.find_one({
             'uuid': upload['chunkUuid']
-        }, projection=['n']).sort('n', pymongo.DESCENDING).limit(1)
-        if cursor.count(True) == 0:
-            n = 0
-        else:
-            n = cursor[0]['n'] + 1
+        }, projection=['n'], sort=[('n', pymongo.DESCENDING)])
+        if lastChunk:
+            # This bit of code will only do anything if there is a discrepancy
+            # between the received count of the upload record and the length of
+            # the file stored as chunks in the database. This code updates the
+            # sha512 state with the difference before reading the bytes sent
+            # from the user.
+            if self.requestOffset(upload) > upload['received']:
+                # This isn't right -- the last received amount may not be a
+                # complete chunk.
+                cursor = self.chunkColl.find({
+                    'uuid': upload['chunkUuid'],
+                    'n': {'$gte': upload['received'] // CHUNK_SIZE}
+                }, projection=['data']).sort('n', pymongo.ASCENDING)
+                for result in cursor:
+                    checksum.update(result['data'])
+        n = lastChunk['n'] + 1 if lastChunk else 0
 
         size = 0
         startingN = n
 
-        while not upload['received']+size > upload['size']:
+        while upload['received']+size < upload['size']:
             data = chunk.read(CHUNK_SIZE)
             if not data:
                 break
@@ -200,14 +267,14 @@ class GridFsAssetstoreAdapter(AbstractAssetstoreAdapter):
         count because in testing mode we are uploading chunks that are smaller
         than the CHUNK_SIZE, which in practice will not work.
         """
-        cursor = self.chunkColl.find({
+        lastChunk = self.chunkColl.find_one({
             'uuid': upload['chunkUuid']
-        }, projection=['n']).sort('n', pymongo.DESCENDING).limit(1)
-        if cursor.count(True) == 0:
+        }, projection=['n'], sort=[('n', pymongo.DESCENDING)])
+
+        if lastChunk is None:
             offset = 0
         else:
-            offset = cursor[0]['n'] * CHUNK_SIZE
-
+            offset = lastChunk['n'] * CHUNK_SIZE
         return max(offset, upload['received'])
 
     def finalizeUpload(self, upload, file):
@@ -287,13 +354,18 @@ class GridFsAssetstoreAdapter(AbstractAssetstoreAdapter):
             'chunkUuid': file['chunkUuid'],
             'assetstoreId': self.assetstore['_id']
         }
-        matching = self.model('file').find(q, limit=2, projection=[])
+        matching = File().find(q, limit=2, projection=[])
         if matching.count(True) == 1:
+            # If we can't reach the database, we return anyway.  A system check
+            # will be necessary to remove the abandoned file.  Since we already
+            # can handle that case, tell Mongo to use a 0 write concern -- we
+            # don't need to know that the chunks have been deleted, and this
+            # can be faster.
             try:
-                self.chunkColl.delete_many({'uuid': file['chunkUuid']})
+                self.chunkColl.with_options(
+                    write_concern=pymongo.WriteConcern(w=0)).delete_many(
+                        {'uuid': file['chunkUuid']})
             except pymongo.errors.AutoReconnect:
-                # we can't reach the database.  Go ahead and return; a system
-                # check will be necessary to remove the abandoned file
                 pass
 
     def cancelUpload(self, upload):
