@@ -36,6 +36,7 @@ import six
 import sys
 import yaml
 import importlib
+import traceback
 
 import pkg_resources
 from pkg_resources import iter_entry_points
@@ -44,9 +45,36 @@ from girder import logprint
 from girder.constants import GIRDER_ROUTE_ID, GIRDER_STATIC_ROUTE_ID, PACKAGE_DIR, ROOT_DIR, \
     ROOT_PLUGINS_PACKAGE, SettingKey
 from girder.models.model_base import ValidationException
-from girder.utility import mail_utils, model_importer
+from girder.models.setting import Setting
+from girder.utility import mail_utils
 
 _pluginWebroots = {}
+_pluginFailureInfo = {}
+
+
+def _recordPluginFailureInfo(plugin, traceback):
+    """
+    Record information when a plugin fails to load.
+
+    :param plugin: The name of the plugin that failed to load.
+    :type plugin: str
+    :param traceback: The traceback of the failure.
+    :type trackback: str
+    """
+    _pluginFailureInfo[plugin] = {
+        'traceback': traceback
+    }
+
+
+def _clearPluginFailureInfo(plugin):
+    """
+    If a plugin loaded, clear any failure information that may have been set by
+    an earlier failed attempt.
+
+    :param plugin: The name of the plugin that loaded.
+    :type plugin: str
+    """
+    _pluginFailureInfo.pop(plugin, None)
 
 
 def loadPlugins(plugins, root, appconf, apiRoot=None, buildDag=True):
@@ -85,18 +113,23 @@ def loadPlugins(plugins, root, appconf, apiRoot=None, buildDag=True):
     for plugin in plugins:
         try:
             root, appconf, apiRoot = loadPlugin(plugin, root, appconf, apiRoot)
+            _clearPluginFailureInfo(plugin=plugin)
             logprint.success('Loaded plugin "%s"' % plugin)
         except Exception:
+            _recordPluginFailureInfo(plugin=plugin, traceback=traceback.format_exc())
             logprint.exception('ERROR: Failed to load plugin "%s":' % plugin)
 
     return root, appconf, apiRoot
 
 
-def getToposortedPlugins(plugins, ignoreMissing=False, keys=('dependencies',)):
+def getToposortedPlugins(plugins=None, ignoreMissing=False, keys=('dependencies',)):
     """
     Given a set of plugins to load, construct the full DAG of required plugins
     to load and yields them in toposorted order.
 
+    :param plugins: A set of plugins that must be in the output list. If you want
+        to toposort all available plugins, omit this parameter.
+    :type plugins: iterable of str
     :param ignoreMissing: Normally if one of the plugins specified does not exist,
         this raises a ValidationError. Set this to False to suppress that and instead
         print an error message and continue.
@@ -104,12 +137,15 @@ def getToposortedPlugins(plugins, ignoreMissing=False, keys=('dependencies',)):
     :param keys: Keys that should be used to determine dependencies.
     :type keys: list of str
     """
-    plugins = set(plugins)
-
     allPlugins = findAllPlugins()
     dag = {}
     visited = set()
 
+    if plugins is None:
+        plugins = set(allPlugins.keys())
+    else:
+        plugins = set(plugins)
+
     def addDeps(plugin):
         if plugin not in allPlugins:
             message = 'Required plugin %s does not exist.' % plugin
@@ -181,44 +217,39 @@ def loadPlugin(name, root, appconf, apiRoot=None):
 
     moduleName = '.'.join((ROOT_PLUGINS_PACKAGE, name))
 
-    if moduleName not in sys.modules:
-        fp = None
-        try:
-            # @todo this query is run for every plugin that's loaded
-            setting = model_importer.ModelImporter().model('setting')
-            routeTable = setting.get(SettingKey.ROUTE_TABLE)
+    fp = None
+    try:
+        # @todo this query is run for every plugin that's loaded
+        routeTable = Setting().get(SettingKey.ROUTE_TABLE)
 
-            info = {
-                'name': name,
-                'config': appconf,
-                'serverRoot': root,
-                'serverRootPath': routeTable[GIRDER_ROUTE_ID],
-                'apiRoot': apiRoot,
-                'staticRoot': routeTable[GIRDER_STATIC_ROUTE_ID],
-                'pluginRootDir': os.path.abspath(pluginDir)
-            }
+        info = {
+            'name': name,
+            'config': appconf,
+            'serverRoot': root,
+            'serverRootPath': routeTable[GIRDER_ROUTE_ID],
+            'apiRoot': apiRoot,
+            'staticRoot': routeTable[GIRDER_STATIC_ROUTE_ID],
+            'pluginRootDir': os.path.abspath(pluginDir)
+        }
 
-            if pluginLoadMethod is None:
-                fp, pathname, description = imp.find_module(
-                    'server', [pluginDir]
-                )
-                module = imp.load_module(moduleName, fp, pathname, description)
-                module.PLUGIN_ROOT_DIR = pluginDir
-                girder.plugins.__dict__[name] = module
-                pluginLoadMethod = getattr(module, 'load', None)
+        if pluginLoadMethod is None:
+            fp, pathname, description = imp.find_module('server', [pluginDir])
+            module = imp.load_module(moduleName, fp, pathname, description)
+            module.PLUGIN_ROOT_DIR = pluginDir
+            girder.plugins.__dict__[name] = module
+            pluginLoadMethod = getattr(module, 'load', None)
 
-            if pluginLoadMethod is not None:
-                sys.modules[moduleName] = module
-                pluginLoadMethod(info)
+        if pluginLoadMethod is not None:
+            sys.modules[moduleName] = module
+            pluginLoadMethod(info)
 
-            root, appconf, apiRoot = (
-                info['serverRoot'], info['config'], info['apiRoot'])
+        root, appconf, apiRoot = (info['serverRoot'], info['config'], info['apiRoot'])
 
-        finally:
-            if fp:
-                fp.close()
+    finally:
+        if fp:
+            fp.close()
 
-        return root, appconf, apiRoot
+    return root, appconf, apiRoot
 
 
 def getPluginDir():
@@ -255,6 +286,9 @@ def findEntryPointPlugins(allPlugins):
                     try:
                         data = json.load(codecs.getreader('utf8')(conf))
                     except ValueError:
+                        _recordPluginFailureInfo(
+                            plugin=entry_point.name,
+                            traceback=traceback.format_exc())
                         logprint.exception(
                             'ERROR: Plugin "%s": plugin.json is not valid '
                             'JSON.' % entry_point.name)
@@ -264,13 +298,29 @@ def findEntryPointPlugins(allPlugins):
                     try:
                         data = yaml.safe_load(conf)
                     except yaml.YAMLError:
+                        _recordPluginFailureInfo(
+                            plugin=entry_point.name,
+                            traceback=traceback.format_exc())
                         logprint.exception(
                             'ERROR: Plugin "%s": plugin.yml is not valid '
                             'YAML.' % entry_point.name)
-        except ImportError:
+        except (ImportError, SystemError):
+            # Fall through and just try to load the entry point below.  If
+            # there is still an error, we'll log it there.
             pass
         if data == {}:
-            data = getattr(entry_point.load(), 'config', {})
+            try:
+                data = getattr(entry_point.load(), 'config', {})
+            except (ImportError, SystemError):
+                # If the plugin failed to load via entrypoint, but is in the
+                # plugins directory, it may still load.  We mark and report the
+                # failure; if it loads later, the failure will be cleared, but
+                # the report is still desired.
+                _recordPluginFailureInfo(
+                    plugin=entry_point.name, traceback=traceback.format_exc())
+                logprint.exception(
+                    'ERROR: Plugin "%s": could not be loaded by entrypoint.' % entry_point.name)
+                continue
         allPlugins[entry_point.name].update(data)
         allPlugins[entry_point.name]['dependencies'] = set(
             allPlugins[entry_point.name]['dependencies'])
@@ -299,6 +349,7 @@ def findAllPlugins():
                 try:
                     data = json.load(conf)
                 except ValueError:
+                    _recordPluginFailureInfo(plugin=plugin, traceback=traceback.format_exc())
                     logprint.exception(
                         'ERROR: Plugin "%s": plugin.json is not valid '
                         'JSON.' % plugin)
@@ -307,6 +358,7 @@ def findAllPlugins():
                 try:
                     data = yaml.safe_load(conf)
                 except yaml.YAMLError:
+                    _recordPluginFailureInfo(plugin=plugin, traceback=traceback.format_exc())
                     logprint.exception(
                         'ERROR: Plugin "%s": plugin.yml is not valid '
                         'YAML.' % plugin)
@@ -397,6 +449,10 @@ def registerPluginWebroot(webroot, name):
     _pluginWebroots[name] = webroot
 
 
+def getPluginFailureInfo():
+    return _pluginFailureInfo
+
+
 class config(object):  # noqa: class name
     """
     Wrap a plugin's ``load`` method appending plugin metadata.
