@@ -18,15 +18,17 @@
 ###############################################################################
 
 import cherrypy
+import functools
 import mako
 import mimetypes
 import os
+import posixpath
 import six
 
 import girder.events
-from girder import constants, logprint, __version__
-from girder.utility import plugin_utilities, model_importer
-from girder.utility import config
+from girder import constants, logprint, __version__, logStdoutStderr
+from girder.models.setting import Setting
+from girder.utility import plugin_utilities, config
 from . import webroot
 
 with open(os.path.join(os.path.dirname(__file__), 'error.mako')) as f:
@@ -41,6 +43,48 @@ def _errorDefault(status, message, *args, **kwargs):
     return mako.template.Template(_errorTemplate).render(status=status, message=message)
 
 
+def _configureStaticRoutes(webroot, plugins, event=None):
+    """
+    Configures static routes for a given webroot.
+
+    This function is also run when the route table setting is modified
+    to allow for dynamically changing static routes at runtime.
+    """
+    # This was triggered by some unrelated setting changing
+    if event is not None and event.info['key'] != constants.SettingKey.ROUTE_TABLE:
+        return
+
+    routeTable = loadRouteTable()
+
+    # If the static route is a URL, leave it alone
+    if '://' in routeTable[constants.GIRDER_STATIC_ROUTE_ID]:
+        apiStaticRoot = routeTable[constants.GIRDER_STATIC_ROUTE_ID]
+        staticRoot = routeTable[constants.GIRDER_STATIC_ROUTE_ID]
+    else:
+        # Make the staticRoot relative to the api_root, if possible.  The api_root
+        # could be relative or absolute, but it needs to be in an absolute form for
+        # relpath to behave as expected.  We always expect the api_root to
+        # contain at least two components, but the reference from static needs to
+        # be from only the first component.
+        apiRootBase = posixpath.split(posixpath.join('/',
+                                                     config.getConfig()['server']['api_root']))[0]
+        apiStaticRoot = posixpath.relpath(routeTable[constants.GIRDER_STATIC_ROUTE_ID],
+                                          apiRootBase)
+        staticRoot = posixpath.relpath(routeTable[constants.GIRDER_STATIC_ROUTE_ID],
+                                       routeTable[constants.GIRDER_ROUTE_ID])
+
+    webroot.updateHtmlVars({
+        'apiRoot': config.getConfig()['server']['api_root'],
+        'staticRoot': staticRoot,
+        'plugins': plugins
+    })
+
+    webroot.api.v1.updateHtmlVars({
+        'apiRoot': config.getConfig()['server']['api_root'],
+        'staticRoot': apiStaticRoot
+    })
+
+
 def configureServer(test=False, plugins=None, curConfig=None):
     """
     Function to setup the cherrypy server. It configures it, but does
@@ -56,8 +100,6 @@ def configureServer(test=False, plugins=None, curConfig=None):
     if curConfig is None:
         curConfig = config.getConfig()
 
-    routeTable = loadRouteTable()
-
     appconf = {
         '/': {
             'request.dispatch': cherrypy.dispatch.MethodDispatcher(),
@@ -103,7 +145,8 @@ def configureServer(test=False, plugins=None, curConfig=None):
             'mode': 'testing',
             'api_root': 'api/v1',
             'static_root': 'static',
-            'api_static_root': '../static'
+            'api_static_root': '../static',
+            'cherrypy_server': True
         }})
 
     mode = curConfig['server']['mode'].lower()
@@ -121,29 +164,14 @@ def configureServer(test=False, plugins=None, curConfig=None):
     cherrypy.engine.subscribe('stop', girder.events.daemon.stop)
 
     if plugins is None:
-        settings = model_importer.ModelImporter().model('setting')
-        plugins = settings.get(constants.SettingKey.PLUGINS_ENABLED, default=())
+        plugins = Setting().get(constants.SettingKey.PLUGINS_ENABLED, default=())
 
     plugins = list(plugin_utilities.getToposortedPlugins(plugins, ignoreMissing=True))
-    root.updateHtmlVars({
-        'apiRoot': curConfig['server']['api_root'],
-        'staticRoot': os.path.relpath(routeTable[constants.GIRDER_STATIC_ROUTE_ID],
-                                      routeTable[constants.GIRDER_ROUTE_ID]),
-        'plugins': plugins
-    })
 
-    # Make the staticRoot relative to the api_root, if possible.  The api_root
-    # could be relative or absolute, but it needs to be in an absolute form for
-    # relpath to behave as expected.  We always expect the api_root to
-    # contain at least two components, but the reference from static needs to
-    # be from only the first component.
-    apiRootBase = os.path.split(os.path.join('/', curConfig['server']['api_root']))[0]
+    _configureStaticRoutes(root, plugins)
 
-    root.api.v1.updateHtmlVars({
-        'apiRoot': curConfig['server']['api_root'],
-        'staticRoot': os.path.relpath(routeTable[constants.GIRDER_STATIC_ROUTE_ID],
-                                      apiRootBase)
-    })
+    girder.events.bind('model.setting.save.after', '_updateStaticRoutesIfModified',
+                       functools.partial(_configureStaticRoutes, root, plugins))
 
     root, appconf, _ = plugin_utilities.loadPlugins(
         plugins, root, appconf, root.api.v1, buildDag=False)
@@ -156,36 +184,25 @@ def loadRouteTable(reconcileRoutes=False):
     Retrieves the route table from Girder and reconciles the state of it with the current
     application state.
 
-    Reconciliation deals with 2 scenarios:
-
-    1. A plugin is no longer active (by being disabled or removed) and the route for the
-       plugin needs to be removed.
-    2. A webroot was added (a new plugin was enabled) and a default route needs to be added.
+    Reconciliation ensures that every enabled plugin has a route by assigning default routes for
+    plugins that have none, such as newly-enabled plugins.
 
     :returns: The non empty routes (as a dict of name -> route) to be mounted by CherryPy
               during Girder's setup phase.
     """
     pluginWebroots = plugin_utilities.getPluginWebroots()
-    setting = model_importer.ModelImporter().model('setting')
-    routeTable = setting.get(constants.SettingKey.ROUTE_TABLE)
-    reservedRoutes = (constants.GIRDER_ROUTE_ID, constants.GIRDER_STATIC_ROUTE_ID)
+    routeTable = Setting().get(constants.SettingKey.ROUTE_TABLE)
 
     def reconcileRouteTable(routeTable):
         hasChanged = False
 
-        # GIRDER_ROUTE_ID is a special route, which can't be removed
-        for name in routeTable.keys():
-            if name not in reservedRoutes and name not in pluginWebroots:
-                del routeTable[name]
-                hasChanged = True
-
         for name in pluginWebroots.keys():
             if name not in routeTable:
                 routeTable[name] = os.path.join('/', name)
                 hasChanged = True
 
         if hasChanged:
-            setting.set(constants.SettingKey.ROUTE_TABLE, routeTable)
+            Setting().set(constants.SettingKey.ROUTE_TABLE, routeTable)
 
         return routeTable
 
@@ -206,6 +223,8 @@ def setup(test=False, plugins=None, curConfig=None):
     :param plugins: List of plugins to enable.
     :param curConfig: The config object to update.
     """
+    logStdoutStderr()
+
     pluginWebroots = plugin_utilities.getPluginWebroots()
     girderWebroot, appconf = configureServer(test, plugins, curConfig)
     routeTable = loadRouteTable(reconcileRoutes=True)
